##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class Decoder(srd.Decoder):
    api_version = 3
    id = 'mlx90614'
    name = 'MLX90614'
    longname = 'Melexis MLX90614'
    desc = 'Melexis MLX90614 infrared thermometer protocol.'
    license = 'gplv2+'
    inputs = ['i2c']
    outputs = []
    tags = ['IC', 'Sensor']
    annotations = (
        ('celsius', 'Temperature in degrees Celsius'),
        ('kelvin', 'Temperature in Kelvin'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IGNORE START REPEAT'
        self.data = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    # Quick hack implementation! This needs to be improved a lot!
    def decode(self, ss, es, data):
        cmd, databyte = data

        # State machine.
        if self.state == 'IGNORE START REPEAT':
            if cmd != 'START REPEAT':
                return
            self.state = 'IGNORE ADDRESS WRITE'
        elif self.state == 'IGNORE ADDRESS WRITE':
            if cmd != 'ADDRESS WRITE':
                return
            self.state = 'GET TEMPERATURE'
        elif self.state == 'GET TEMPERATURE':
            if cmd != 'DATA WRITE':
                return
            if len(self.data) == 0:
                self.data.append(databyte)
                self.ss = ss
            elif len(self.data) == 1:
                self.data.append(databyte)
                self.es = es
            else:
                kelvin = (self.data[0] | (self.data[1] << 8)) * 0.02
                celsius = kelvin - 273.15
                self.putx([0, ['Temperature: %3.2f °C' % celsius]])
                self.putx([1, ['Temperature: %3.2f K' % kelvin]])
                self.state = 'IGNORE START REPEAT'
                self.data = []



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012-2016 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
import calendar
from common.srdhelper import bcd2int

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'dcf77'
    name = 'DCF77'
    longname = 'DCF77 time protocol'
    desc = 'European longwave time signal (77.5kHz carrier signal).'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Clock/timing']
    channels = (
        {'id': 'data', 'name': 'DATA', 'desc': 'DATA line'},
    )
    annotations = (
        ('start-of-minute', 'Start of minute'),
        ('special-bits', 'Special bits (civil warnings, weather forecast)'),
        ('call-bit', 'Call bit'),
        ('summer-time', 'Summer time announcement'),
        ('cest', 'CEST bit'),
        ('cet', 'CET bit'),
        ('leap-second', 'Leap second bit'),
        ('start-of-time', 'Start of encoded time'),
        ('minute', 'Minute'),
        ('minute-parity', 'Minute parity bit'),
        ('hour', 'Hour'),
        ('hour-parity', 'Hour parity bit'),
        ('day', 'Day of month'),
        ('day-of-week', 'Day of week'),
        ('month', 'Month'),
        ('year', 'Year'),
        ('date-parity', 'Date parity bit'),
        ('raw-bits', 'Raw bits'),
        ('unknown-bits', 'Unknown bits'),
        ('warnings', 'Human-readable warnings'),
    )
    annotation_rows = (
        ('bits', 'Bits', (17, 18)),
        ('fields', 'Fields', tuple(range(0, 16 + 1))),
        ('warnings', 'Warnings', (19,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.state = 'WAIT FOR RISING EDGE'
        self.ss_bit = self.ss_bit_old = self.es_bit = self.ss_block = 0
        self.datebits = []
        self.bitcount = 0 # Counter for the DCF77 bits (0..58)
        self.dcf77_bitnumber_is_known = 0

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def putx(self, data):
        # Annotation for a single DCF77 bit.
        self.put(self.ss_bit, self.es_bit, self.out_ann, data)

    def putb(self, data):
        # Annotation for a multi-bit DCF77 field.
        self.put(self.ss_block, self.samplenum, self.out_ann, data)

    # TODO: Which range to use? Only the 100ms/200ms or full second?
    def handle_dcf77_bit(self, bit):
        c = self.bitcount

        # Create one annotation for each DCF77 bit (containing the 0/1 value).
        # Use 'Unknown DCF77 bit x: val' if we're not sure yet which of the
        # 0..58 bits it is (because we haven't seen a 'new minute' marker yet).
        # Otherwise, use 'DCF77 bit x: val'.
        s = 'B' if self.dcf77_bitnumber_is_known else 'Unknown b'
        ann = 17 if self.dcf77_bitnumber_is_known else 18
        self.putx([ann, ['%sit %d: %d' % (s, c, bit), '%d' % bit]])

        # If we're not sure yet which of the 0..58 DCF77 bits we have, return.
        # We don't want to decode bogus data.
        if not self.dcf77_bitnumber_is_known:
            return

        # Collect bits 36-58, we'll need them for a parity check later.
        if c in range(36, 58 + 1):
            self.datebits.append(bit)

        # Output specific "decoded" annotations for the respective DCF77 bits.
        if c == 0:
            # Start of minute: DCF bit 0.
            if bit == 0:
                self.putx([0, ['Start of minute (always 0)',
                               'Start of minute', 'SoM']])
            else:
                self.putx([19, ['Start of minute != 0', 'SoM != 0']])
        elif c in range(1, 14 + 1):
            # Special bits (civil warnings, weather forecast): DCF77 bits 1-14.
            if c == 1:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 1))
            if c == 14:
                s = '{:014b}'.format(self.tmp)
                self.putb([1, ['Special bits: %s' % s, 'SB: %s' % s]])
        elif c == 15:
            s = '' if (bit == 1) else 'not '
            self.putx([2, ['Call bit: %sset' % s, 'CB: %sset' % s]])
            # TODO: Previously this bit indicated use of the backup antenna.
        elif c == 16:
            s = '' if (bit == 1) else 'not '
            x = 'yes' if (bit == 1) else 'no'
            self.putx([3, ['Summer time announcement: %sactive' % s,
                           'Summer time: %sactive' % s,
                           'Summer time: %s' % x, 'ST: %s' % x]])
        elif c == 17:
            s = '' if (bit == 1) else 'not '
            x = 'yes' if (bit == 1) else 'no'
            self.putx([4, ['CEST: %sin effect' % s, 'CEST: %s' % x]])
        elif c == 18:
            s = '' if (bit == 1) else 'not '
            x = 'yes' if (bit == 1) else 'no'
            self.putx([5, ['CET: %sin effect' % s, 'CET: %s' % x]])
        elif c == 19:
            s = '' if (bit == 1) else 'not '
            x = 'yes' if (bit == 1) else 'no'
            self.putx([6, ['Leap second announcement: %sactive' % s,
                           'Leap second: %sactive' % s,
                           'Leap second: %s' % x, 'LS: %s' % x]])
        elif c == 20:
            # Start of encoded time: DCF bit 20.
            if bit == 1:
                self.putx([7, ['Start of encoded time (always 1)',
                               'Start of encoded time', 'SoeT']])
            else:
                self.putx([19, ['Start of encoded time != 1', 'SoeT != 1']])
        elif c in range(21, 27 + 1):
            # Minutes (0-59): DCF77 bits 21-27 (BCD format).
            if c == 21:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 21))
            if c == 27:
                m = bcd2int(self.tmp)
                self.putb([8, ['Minutes: %d' % m, 'Min: %d' % m]])
        elif c == 28:
            # Even parity over minute bits (21-28): DCF77 bit 28.
            self.tmp |= (bit << (c - 21))
            parity = bin(self.tmp).count('1')
            s = 'OK' if ((parity % 2) == 0) else 'INVALID!'
            self.putx([9, ['Minute parity: %s' % s, 'Min parity: %s' % s]])
        elif c in range(29, 34 + 1):
            # Hours (0-23): DCF77 bits 29-34 (BCD format).
            if c == 29:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 29))
            if c == 34:
                self.putb([10, ['Hours: %d' % bcd2int(self.tmp)]])
        elif c == 35:
            # Even parity over hour bits (29-35): DCF77 bit 35.
            self.tmp |= (bit << (c - 29))
            parity = bin(self.tmp).count('1')
            s = 'OK' if ((parity % 2) == 0) else 'INVALID!'
            self.putx([11, ['Hour parity: %s' % s]])
        elif c in range(36, 41 + 1):
            # Day of month (1-31): DCF77 bits 36-41 (BCD format).
            if c == 36:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 36))
            if c == 41:
                self.putb([12, ['Day: %d' % bcd2int(self.tmp)]])
        elif c in range(42, 44 + 1):
            # Day of week (1-7): DCF77 bits 42-44 (BCD format).
            # A value of 1 means Monday, 7 means Sunday.
            if c == 42:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 42))
            if c == 44:
                d = bcd2int(self.tmp)
                try:
                    dn = calendar.day_name[d - 1] # day_name[0] == Monday
                    self.putb([13, ['Day of week: %d (%s)' % (d, dn),
                                    'DoW: %d (%s)' % (d, dn)]])
                except IndexError:
                    self.putb([19, ['Day of week: %d (%s)' % (d, 'invalid'),
                                    'DoW: %d (%s)' % (d, 'inv')]])
        elif c in range(45, 49 + 1):
            # Month (1-12): DCF77 bits 45-49 (BCD format).
            if c == 45:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 45))
            if c == 49:
                m = bcd2int(self.tmp)
                try:
                    mn = calendar.month_name[m] # month_name[1] == January
                    self.putb([14, ['Month: %d (%s)' % (m, mn),
                                    'Mon: %d (%s)' % (m, mn)]])
                except IndexError:
                    self.putb([19, ['Month: %d (%s)' % (m, 'invalid'),
                                    'Mon: %d (%s)' % (m, 'inv')]])
        elif c in range(50, 57 + 1):
            # Year (0-99): DCF77 bits 50-57 (BCD format).
            if c == 50:
                self.tmp = bit
                self.ss_block = self.ss_bit
            else:
                self.tmp |= (bit << (c - 50))
            if c == 57:
                self.putb([15, ['Year: %d' % bcd2int(self.tmp)]])
        elif c == 58:
            # Even parity over date bits (36-58): DCF77 bit 58.
            parity = self.datebits.count(1)
            s = 'OK' if ((parity % 2) == 0) else 'INVALID!'
            self.putx([16, ['Date parity: %s' % s, 'DP: %s' % s]])
            self.datebits = []
        else:
            self.putx([19, ['Invalid DCF77 bit: %d' % c,
                            'Invalid bit: %d' % c, 'Inv: %d' % c]])

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        while True:
            if self.state == 'WAIT FOR RISING EDGE':
                # Wait until the next rising edge occurs.
                self.wait({0: 'r'})

                # Save the sample number where the DCF77 bit begins.
                self.ss_bit = self.samplenum

                # Calculate the length (in ms) between two rising edges.
                len_edges = self.ss_bit - self.ss_bit_old
                len_edges_ms = int((len_edges / self.samplerate) * 1000)

                # The time between two rising edges is usually around 1000ms.
                # For DCF77 bit 59, there is no rising edge at all, i.e. the
                # time between DCF77 bit 59 and DCF77 bit 0 (of the next
                # minute) is around 2000ms. Thus, if we see an edge with a
                # 2000ms distance to the last one, this edge marks the
                # beginning of a new minute (and DCF77 bit 0 of that minute).
                if len_edges_ms in range(1600, 2400 + 1):
                    self.bitcount = 0
                    self.ss_bit_old = self.ss_bit
                    self.dcf77_bitnumber_is_known = 1

                self.ss_bit_old = self.ss_bit
                self.state = 'GET BIT'

            elif self.state == 'GET BIT':
                # Wait until the next falling edge occurs.
                self.wait({0: 'f'})

                # Save the sample number where the DCF77 bit ends.
                self.es_bit = self.samplenum

                # Calculate the length (in ms) of the current high period.
                len_high = self.samplenum - self.ss_bit
                len_high_ms = int((len_high / self.samplerate) * 1000)

                # If the high signal was 100ms long, that encodes a 0 bit.
                # If it was 200ms long, that encodes a 1 bit.
                if len_high_ms in range(40, 160 + 1):
                    bit = 0
                elif len_high_ms in range(161, 260 + 1):
                    bit = 1
                else:
                    bit = -1

                if bit in (0, 1):
                    self.handle_dcf77_bit(bit)
                    self.bitcount += 1
                else:
                    self.putx([19, ['Invalid bit timing', 'Inv timing', 'Inv']])

                self.state = 'WAIT FOR RISING EDGE'



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Jorge Solla Rubiales <jorgesolla@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.
##

logical_adresses = [
    'TV',
    'Recording_1',
    'Recording_2',
    'Tuner_1',
    'Playback_1',
    'AudioSystem',
    'Tuner2',
    'Tuner3',
    'Playback_2',
    'Recording_3',
    'Tuner_4',
    'Playback_3',
    'Backup_1',
    'Backup_2',
    'FreeUse',
]

# List taken from LibCEC.
opcodes = {
    0x82: 'ACTIVE_SOURCE',
    0x04: 'IMAGE_VIEW_ON',
    0x0D: 'TEXT_VIEW_ON',
    0x9D: 'INACTIVE_SOURCE',
    0x85: 'REQUEST_ACTIVE_SOURCE',
    0x80: 'ROUTING_CHANGE',
    0x81: 'ROUTING_INFORMATION',
    0x86: 'SET_STREAM_PATH',
    0x36: 'STANDBY',
    0x0B: 'RECORD_OFF',
    0x09: 'RECORD_ON',
    0x0A: 'RECORD_STATUS',
    0x0F: 'RECORD_TV_SCREEN',
    0x33: 'CLEAR_ANALOGUE_TIMER',
    0x99: 'CLEAR_DIGITAL_TIMER',
    0xA1: 'CLEAR_EXTERNAL_TIMER',
    0x34: 'SET_ANALOGUE_TIMER',
    0x97: 'SET_DIGITAL_TIMER',
    0xA2: 'SET_EXTERNAL_TIMER',
    0x67: 'SET_TIMER_PROGRAM_TITLE',
    0x43: 'TIMER_CLEARED_STATUS',
    0x35: 'TIMER_STATUS',
    0x9E: 'CEC_VERSION',
    0x9F: 'GET_CEC_VERSION',
    0x83: 'GIVE_PHYSICAL_ADDRESS',
    0x91: 'GET_MENU_LANGUAGE',
    0x84: 'REPORT_PHYSICAL_ADDRESS',
    0x32: 'SET_MENU_LANGUAGE',
    0x42: 'DECK_CONTROL',
    0x1B: 'DECK_STATUS',
    0x1A: 'GIVE_DECK_STATUS',
    0x41: 'PLAY',
    0x08: 'GIVE_TUNER_DEVICE_STATUS',
    0x92: 'SELECT_ANALOGUE_SERVICE',
    0x93: 'SELECT_DIGITAL_SERVICE',
    0x07: 'TUNER_DEVICE_STATUS',
    0x06: 'TUNER_STEP_DECREMENT',
    0x05: 'TUNER_STEP_INCREMENT',
    0x87: 'DEVICE_VENDOR_ID',
    0x8C: 'GIVE_DEVICE_VENDOR_ID',
    0x89: 'VENDOR_COMMAND',
    0xA0: 'VENDOR_COMMAND_WITH_ID',
    0x8A: 'VENDOR_REMOTE_BUTTON_DOWN',
    0x8B: 'VENDOR_REMOTE_BUTTON_UP',
    0x64: 'SET_OSD_STRING',
    0x46: 'GIVE_OSD_NAME',
    0x47: 'SET_OSD_NAME',
    0x8D: 'MENU_REQUEST',
    0x8E: 'MENU_STATUS',
    0x44: 'USER_CONTROL_PRESSED',
    0x45: 'USER_CONTROL_RELEASE',
    0x8F: 'GIVE_DEVICE_POWER_STATUS',
    0x90: 'REPORT_POWER_STATUS',
    0x00: 'FEATURE_ABORT',
    0xFF: 'ABORT',
    0x71: 'GIVE_AUDIO_STATUS',
    0x7D: 'GIVE_SYSTEM_AUDIO_MODE_STATUS',
    0x7A: 'REPORT_AUDIO_STATUS',
    0x72: 'SET_SYSTEM_AUDIO_MODE',
    0x70: 'SYSTEM_AUDIO_MODE_REQUEST',
    0x7E: 'SYSTEM_AUDIO_MODE_STATUS',
    0x9A: 'SET_AUDIO_RATE',
}

def resolve_logical_address(id_, is_initiator):
    if id_ < 0 or id_ > 0x0F:
        return 'Invalid'

    # Special handling of 0x0F.
    if id_ == 0x0F:
        return 'Unregistered' if is_initiator else 'Broadcast'

    return logical_adresses[id_]

def decode_header(header):
    src = (header & 0xF0) >> 4
    dst = (header & 0x0F)
    return (resolve_logical_address(src, 1), resolve_logical_address(dst, 0))



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Karl Palsson <karlp@tweak.net.au>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

regs = {
    0: 'Product_ID',
    1: 'Revision_ID',
    2: 'Motion',
    3: 'Delta_X',
    4: 'Delta_Y',
    5: 'SQUAL',
    6: 'Shutter_Upper',
    7: 'Shutter_Lower',
    8: 'Maximum_Pixel',
    9: 'Pixel_Sum',
    0xa: 'Minimum_Pixel',
    0xb: 'Pixel_Grab',
    0xd: 'Mouse_Control',
    0x3a: 'Chip_Reset',
    0x3f: 'Inv_Rev_ID',
    0x63: 'Motion_Burst',
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'adns5020'
    name = 'ADNS-5020'
    longname = 'Avago ADNS-5020'
    desc = 'Bidirectional optical mouse sensor protocol.'
    license = 'gplv2+'
    inputs = ['spi']
    outputs = []
    tags = ['IC', 'PC', 'Sensor']
    annotations = (
        ('read', 'Register read commands'),
        ('write', 'Register write commands'),
        ('warning', 'Warnings'),
    )
    annotation_rows = (
        ('read', 'Read', (0,)),
        ('write', 'Write', (1,)),
        ('warnings', 'Warnings', (2,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.ss_cmd, self.es_cmd = 0, 0
        self.mosi_bytes = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss_cmd, self.es_cmd, self.out_ann, data)

    def put_warn(self, pos, msg):
        self.put(pos[0], pos[1], self.out_ann, [2, [msg]])

    def decode(self, ss, es, data):
        ptype = data[0]
        if ptype == 'CS-CHANGE':
            # If we transition high mid-stream, toss out our data and restart.
            cs_old, cs_new = data[1:]
            if cs_old is not None and cs_old == 0 and cs_new == 1:
                if len(self.mosi_bytes) not in [0, 2]:
                    self.put_warn([self.ss_cmd, es], 'Misplaced CS#!')
                    self.mosi_bytes = []
            return

        # Don't care about anything else.
        if ptype != 'DATA':
            return
        mosi, miso = data[1:]

        self.ss, self.es = ss, es

        if len(self.mosi_bytes) == 0:
            self.ss_cmd = ss
        self.mosi_bytes.append(mosi)

        # Writes/reads are mostly two transfers (burst mode is different).
        if len(self.mosi_bytes) != 2:
            return

        self.es_cmd = es
        cmd, arg = self.mosi_bytes
        write = cmd & 0x80
        reg = cmd & 0x7f
        reg_desc = regs.get(reg, 'Reserved %#x' % reg)
        if reg > 0x63:
            reg_desc = 'Unknown'
        if write:
            self.putx([1, ['%s: %#x' % (reg_desc, arg)]])
        else:
            self.putx([0, ['%s: %d' % (reg_desc, arg)]])

        self.mosi_bytes = []



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Steve R <steversig@virginmedia.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
This PD decodes the remote control protocol which is frequently used
within key fobs and power socket remotes.

They contain encoding chips like the PT2262 which converts the button
pressed and address settings into a series of pulses which is then
transmitted over whatever frequency and modulation that the designer
chooses. These devices operate at a number of frequencies including 433MHz.

This PD should also decode the HX2262 and SC5262 which are equivalents.

The decoder also contains some additional decoding for a Maplin L95AR
remote control and will turn the received signal into which button was
pressed and what the address code DIP switches are set to.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Daniel Elstner <daniel.kitta@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from functools import reduce
from .tables import instr_table_by_prefix
import string

class Ann:
    ADDR, MEMRD, MEMWR, IORD, IOWR, INSTR, ROP, WOP, WARN = range(9)
class Row:
    ADDRBUS, DATABUS, INSTRUCTIONS, OPERANDS, WARNINGS = range(5)
class Pin:
    D0, D7 = 0, 7
    M1, RD, WR, MREQ, IORQ = range(8, 13)
    A0, A15 = 13, 28
class Cycle:
    NONE, MEMRD, MEMWR, IORD, IOWR, FETCH, INTACK = range(7)

# Provide custom format type 'H' for hexadecimal output
# with leading decimal digit (assembler syntax).
class AsmFormatter(string.Formatter):
    def format_field(self, value, format_spec):
        if format_spec.endswith('H'):
            result = format(value, format_spec[:-1] + 'X')
            return result if result[0] in string.digits else '0' + result
        else:
            return format(value, format_spec)

formatter = AsmFormatter()

ann_data_cycle_map = {
    Cycle.MEMRD:  Ann.MEMRD,
    Cycle.MEMWR:  Ann.MEMWR,
    Cycle.IORD:   Ann.IORD,
    Cycle.IOWR:   Ann.IOWR,
    Cycle.FETCH:  Ann.MEMRD,
    Cycle.INTACK: Ann.IORD,
}

def reduce_bus(bus):
    if 0xFF in bus:
        return None # unassigned bus channels
    else:
        return reduce(lambda a, b: (a << 1) | b, reversed(bus))

def signed_byte(byte):
    return byte if byte < 128 else byte - 256

class Decoder(srd.Decoder):
    api_version = 3
    id       = 'z80'
    name     = 'Z80'
    longname = 'Zilog Z80 CPU'
    desc     = 'Zilog Z80 microprocessor disassembly.'
    license  = 'gplv3+'
    inputs   = ['logic']
    outputs  = []
    tags     = ['Retro computing']
    channels = tuple({
            'id': 'd%d' % i,
            'name': 'D%d' % i,
            'desc': 'Data bus line %d' % i
            } for i in range(8)
    ) + (
        {'id': 'm1', 'name': '/M1', 'desc': 'Machine cycle 1'},
        {'id': 'rd', 'name': '/RD', 'desc': 'Memory or I/O read'},
        {'id': 'wr', 'name': '/WR', 'desc': 'Memory or I/O write'},
    )
    optional_channels = (
        {'id': 'mreq', 'name': '/MREQ', 'desc': 'Memory request'},
        {'id': 'iorq', 'name': '/IORQ', 'desc': 'I/O request'},
    ) + tuple({
        'id': 'a%d' % i,
        'name': 'A%d' % i,
        'desc': 'Address bus line %d' % i
        } for i in range(16)
    )
    annotations = (
        ('addr',  'Memory or I/O address'),
        ('memrd', 'Byte read from memory'),
        ('memwr', 'Byte written to memory'),
        ('iord',  'Byte read from I/O port'),
        ('iowr',  'Byte written to I/O port'),
        ('instr', 'Z80 CPU instruction'),
        ('rop',   'Value of input operand'),
        ('wop',   'Value of output operand'),
        ('warn',  'Warning message'),
    )
    annotation_rows = (
        ('addrbus', 'Address bus', (Ann.ADDR,)),
        ('databus', 'Data bus', (Ann.MEMRD, Ann.MEMWR, Ann.IORD, Ann.IOWR)),
        ('instructions', 'Instructions', (Ann.INSTR,)),
        ('operands', 'Operands', (Ann.ROP, Ann.WOP)),
        ('warnings', 'Warnings', (Ann.WARN,))
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.prev_cycle = Cycle.NONE
        self.op_state   = self.state_IDLE

    def start(self):
        self.out_ann    = self.register(srd.OUTPUT_ANN)
        self.bus_data   = None
        self.samplenum  = None
        self.addr_start = None
        self.data_start = None
        self.dasm_start = None
        self.pend_addr  = None
        self.pend_data  = None
        self.ann_data   = None
        self.ann_dasm   = None
        self.prev_cycle = Cycle.NONE
        self.op_state   = self.state_IDLE
        self.instr_len  = 0

    def decode(self):
        while True:
            # TODO: Come up with more appropriate self.wait() conditions.
            pins = self.wait()
            cycle = Cycle.NONE
            if pins[Pin.MREQ] != 1: # default to asserted
                if pins[Pin.RD] == 0:
                    cycle = Cycle.FETCH if pins[Pin.M1] == 0 else Cycle.MEMRD
                elif pins[Pin.WR] == 0:
                    cycle = Cycle.MEMWR
            elif pins[Pin.IORQ] == 0: # default to not asserted
                if pins[Pin.M1] == 0:
                    cycle = Cycle.INTACK
                elif pins[Pin.RD] == 0:
                    cycle = Cycle.IORD
                elif pins[Pin.WR] == 0:
                    cycle = Cycle.IOWR

            if cycle != Cycle.NONE:
                self.bus_data = reduce_bus(pins[Pin.D0:Pin.D7+1])
            if cycle != self.prev_cycle:
                if self.prev_cycle == Cycle.NONE:
                    self.on_cycle_begin(reduce_bus(pins[Pin.A0:Pin.A15+1]))
                elif cycle == Cycle.NONE:
                    self.on_cycle_end()
                else:
                    self.on_cycle_trans()
            self.prev_cycle = cycle

    def on_cycle_begin(self, bus_addr):
        if self.pend_addr is not None:
            self.put_text(self.addr_start, Ann.ADDR,
                          '{:04X}'.format(self.pend_addr))
        self.addr_start = self.samplenum
        self.pend_addr  = bus_addr

    def on_cycle_end(self):
        self.instr_len += 1
        self.op_state = self.op_state()
        if self.ann_dasm is not None:
            self.put_disasm()
        if self.op_state == self.state_RESTART:
            self.op_state = self.state_IDLE()

        if self.ann_data is not None:
            self.put_text(self.data_start, self.ann_data,
                          '{:02X}'.format(self.pend_data))
        self.data_start = self.samplenum
        self.pend_data  = self.bus_data
        self.ann_data   = ann_data_cycle_map[self.prev_cycle]

    def on_cycle_trans(self):
        self.put_text(self.samplenum - 1, Ann.WARN,
                      'Illegal transition between control states')
        self.pend_addr = None
        self.ann_data  = None
        self.ann_dasm  = None

    def put_disasm(self):
        text = formatter.format(self.mnemonic, r=self.arg_reg, d=self.arg_dis,
                                j=self.arg_dis+self.instr_len, i=self.arg_imm,
                                ro=self.arg_read, wo=self.arg_write)
        self.put_text(self.dasm_start, self.ann_dasm, text)
        self.ann_dasm   = None
        self.dasm_start = self.samplenum

    def put_text(self, ss, ann_idx, ann_text):
        self.put(ss, self.samplenum, self.out_ann, [ann_idx, [ann_text]])

    def state_RESTART(self):
        return self.state_IDLE

    def state_IDLE(self):
        if self.prev_cycle != Cycle.FETCH:
            return self.state_IDLE
        self.want_dis   = 0
        self.want_imm   = 0
        self.want_read  = 0
        self.want_write = 0
        self.want_wr_be = False
        self.op_repeat  = False
        self.arg_dis    = 0
        self.arg_imm    = 0
        self.arg_read   = 0
        self.arg_write  = 0
        self.arg_reg    = ''
        self.mnemonic   = ''
        self.instr_pend = False
        self.read_pend  = False
        self.write_pend = False
        self.dasm_start = self.samplenum
        self.op_prefix  = 0
        self.instr_len  = 0
        if self.bus_data in (0xCB, 0xED, 0xDD, 0xFD):
            return self.state_PRE1
        else:
            return self.state_OPCODE

    def state_PRE1(self):
        if self.prev_cycle != Cycle.FETCH:
            self.mnemonic = 'Prefix not followed by fetch'
            self.ann_dasm = Ann.WARN
            return self.state_RESTART
        self.op_prefix = self.pend_data
        if self.op_prefix in (0xDD, 0xFD):
            if self.bus_data == 0xCB:
                return self.state_PRE2
            if self.bus_data in (0xDD, 0xED, 0xFD):
                return self.state_PRE1
        return self.state_OPCODE

    def state_PRE2(self):
        if self.prev_cycle != Cycle.MEMRD:
            self.mnemonic = 'Missing displacement'
            self.ann_dasm = Ann.WARN
            return self.state_RESTART
        self.op_prefix = (self.op_prefix << 8) | self.pend_data
        return self.state_PREDIS

    def state_PREDIS(self):
        if self.prev_cycle != Cycle.MEMRD:
            self.mnemonic = 'Missing opcode'
            self.ann_dasm = Ann.WARN
            return self.state_RESTART
        self.arg_dis = signed_byte(self.pend_data)
        return self.state_OPCODE

    def state_OPCODE(self):
        (table, self.arg_reg) = instr_table_by_prefix[self.op_prefix]
        self.op_prefix = 0
        instruction = table.get(self.pend_data, None)
        if instruction is None:
            self.mnemonic = 'Invalid instruction'
            self.ann_dasm = Ann.WARN
            return self.state_RESTART
        (self.want_dis, self.want_imm, self.want_read, want_write,
                self.op_repeat, self.mnemonic) = instruction
        self.want_write = abs(want_write)
        self.want_wr_be = (want_write < 0)
        if self.want_dis > 0:
            return self.state_POSTDIS
        if self.want_imm > 0:
            return self.state_IMM1
        self.ann_dasm = Ann.INSTR
        if self.want_read > 0 and self.prev_cycle in (Cycle.MEMRD, Cycle.IORD):
            return self.state_ROP1
        if self.want_write > 0 and self.prev_cycle in (Cycle.MEMWR, Cycle.IOWR):
            return self.state_WOP1
        return self.state_RESTART

    def state_POSTDIS(self):
        self.arg_dis = signed_byte(self.pend_data)
        if self.want_imm > 0:
            return self.state_IMM1
        self.ann_dasm = Ann.INSTR
        if self.want_read > 0 and self.prev_cycle in (Cycle.MEMRD, Cycle.IORD):
            return self.state_ROP1
        if self.want_write > 0 and self.prev_cycle in (Cycle.MEMWR, Cycle.IOWR):
            return self.state_WOP1
        return self.state_RESTART

    def state_IMM1(self):
        self.arg_imm = self.pend_data
        if self.want_imm > 1:
            return self.state_IMM2
        self.ann_dasm = Ann.INSTR
        if self.want_read > 0 and self.prev_cycle in (Cycle.MEMRD, Cycle.IORD):
            return self.state_ROP1
        if self.want_write > 0 and self.prev_cycle in (Cycle.MEMWR, Cycle.IOWR):
            return self.state_WOP1
        return self.state_RESTART

    def state_IMM2(self):
        self.arg_imm |= self.pend_data << 8
        self.ann_dasm = Ann.INSTR
        if self.want_read > 0 and self.prev_cycle in (Cycle.MEMRD, Cycle.IORD):
            return self.state_ROP1
        if self.want_write > 0 and self.prev_cycle in (Cycle.MEMWR, Cycle.IOWR):
            return self.state_WOP1
        return self.state_RESTART

    def state_ROP1(self):
        self.arg_read = self.pend_data
        if self.want_read < 2:
            self.mnemonic = '{ro:02X}'
            self.ann_dasm = Ann.ROP
        if self.want_write > 0:
            return self.state_WOP1
        if self.want_read > 1:
            return self.state_ROP2
        if self.op_repeat and self.prev_cycle in (Cycle.MEMRD, Cycle.IORD):
            return self.state_ROP1
        return self.state_RESTART

    def state_ROP2(self):
        self.arg_read |= self.pend_data << 8
        self.mnemonic = '{ro:04X}'
        self.ann_dasm = Ann.ROP
        if self.want_write > 0 and self.prev_cycle in (Cycle.MEMWR, Cycle.IOWR):
            return self.state_WOP1
        return self.state_RESTART

    def state_WOP1(self):
        self.arg_write = self.pend_data
        if self.want_read > 1:
            return self.state_ROP2
        if self.want_write > 1:
            return self.state_WOP2
        self.mnemonic = '{wo:02X}'
        self.ann_dasm = Ann.WOP
        if self.want_read > 0 and self.op_repeat and \
                self.prev_cycle in (Cycle.MEMRD, Cycle.IORD):
            return self.state_ROP1
        return self.state_RESTART

    def state_WOP2(self):
        if self.want_wr_be:
            self.arg_write = (self.arg_write << 8) | self.pend_data
        else:
            self.arg_write |= self.pend_data << 8
        self.mnemonic = '{wo:04X}'
        self.ann_dasm = Ann.WOP
        return self.state_RESTART



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Stephan Thiele <stephan.thiele@mailbox.org>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class Decoder(srd.Decoder):
    api_version = 3
    id = 'nes_gamepad'
    name = 'NES gamepad'
    longname = 'Nintendo Entertainment System gamepad'
    desc = 'NES gamepad button states.'
    license = 'gplv2+'
    inputs = ['spi']
    outputs = []
    tags = ['Retro computing']
    options = (
        # Currently only the standard controller is supported. This might be
        # extended by special controllers like the Nintendo Zapper light gun.
        {'id': 'variant', 'desc': 'Gamepad variant',
            'default': 'Standard gamepad', 'values': ('Standard gamepad',)},
    )
    annotations = (
        ('button', 'Button states'),
        ('no-press', 'No button press'),
        ('not-connected', 'Gamepad unconnected')
    )
    annotation_rows = (
        ('buttons', 'Button states', (0,)),
        ('no-press', 'No button press', (1,)),
        ('not-connected', 'Gamepad unconnected', (2,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.variant = None
        self.ss_block = None
        self.es_block = None

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.variant = self.options['variant']

    def putx(self, data):
        self.put(self.ss_block, self.es_block, self.out_ann, data)

    def handle_data(self, value):
        if value == 0xFF:
          self.putx([1, ['No button is pressed']])
          return

        if value == 0x00:
          self.putx([2, ['Gamepad is not connected']])
          return

        buttons = [
            'A',
            'B',
            'Select',
            'Start',
            'North',
            'South',
            'West',
            'East'
        ]

        bits = format(value, '08b')
        button_str = ''

        for b in enumerate(bits):
            button_index = b[0]
            button_is_pressed = b[1] == '0'

            if button_is_pressed:
                if button_str != '':
                    button_str += ' + '
                button_str += buttons[button_index]

        self.putx([0, ['%s' % button_str]])

    def decode(self, ss, es, data):
        ptype, mosi, miso = data
        self.ss_block, self.es_block = ss, es

        if ptype != 'DATA':
          return

        self.handle_data(miso)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 fenugrec <fenugrec@users.sourceforge.net>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
This protocol decoder de-multiplexes Intel MCS-48 (8039, 8048, etc.) external
program memory accesses.

This requires 14 channels: 8 for D0-D7 (data and lower 8 bits of address),
4 for A8-A11 (output on port P2), ALE and PSEN.

An optional A12 is supported, which may be an arbitrary I/O pin driven by
software (use case is dumping ROM of an HP 3478A).
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Mike Jagdis <mjagdis@eris-associates.co.uk>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import math
import sigrokdecode as srd

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'swim'
    name = 'SWIM'
    longname = 'STM8 SWIM bus'
    desc = 'STM8 Single Wire Interface Module (SWIM) protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Debug/trace']
    options = (
        {'id': 'debug', 'desc': 'Debug', 'default': 'no', 'values': ('yes', 'no') },
    )
    channels = (
        {'id': 'swim', 'name': 'SWIM', 'desc': 'SWIM data line'},
    )
    annotations = (
        ('bit', 'Bit'),
        ('enterseq', 'SWIM enter sequence'),
        ('start-host', 'Start bit (host)'),
        ('start-target', 'Start bit (target)'),
        ('parity', 'Parity bit'),
        ('ack', 'Acknowledgement'),
        ('nack', 'Negative acknowledgement'),
        ('byte-write', 'Byte write'),
        ('byte-read', 'Byte read'),
        ('cmd-unknown', 'Unknown SWIM command'),
        ('cmd', 'SWIM command'),
        ('bytes', 'Byte count'),
        ('address', 'Address'),
        ('data-write', 'Data write'),
        ('data-read', 'Data read'),
        ('debug', 'Debug'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('framing', 'Framing', (2, 3, 4, 5, 6, 7, 8)),
        ('protocol', 'Protocol', (1, 9, 10, 11, 12, 13, 14)),
        ('debug', 'Debug', (15,)),
    )
    binary = (
        ('tx', 'Dump of data written to target'),
        ('rx', 'Dump of data read from target'),
    )

    def __init__(self):
        # SWIM clock for the target is normally HSI/2 where HSI is 8MHz +- 5%
        # although the divisor can be removed by setting the SWIMCLK bit in
        # the CLK_SWIMCCR register. There is no standard for the host so we
        # will be generous and assume it is using an 8MHz +- 10% oscillator.
        # We do not need to be accurate. We just need to avoid treating enter
        # sequence pulses as bits. A synchronization frame will cause this
        # to be adjusted.
        self.HSI = 8000000
        self.HSI_min = self.HSI * 0.9
        self.HSI_max = self.HSI * 1.1
        self.swim_clock = self.HSI_min / 2

        self.eseq_edge = [[-1, None], [-1, None]]
        self.eseq_pairnum = 0
        self.eseq_pairstart = None

        self.reset()

    def reset(self):
        self.bit_edge = [[-1, None], [-1, None]]
        self.bit_maxlen = -1
        self.bitseq_len = 0
        self.bitseq_end = None
        self.proto_state = 'CMD'

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def adjust_timings(self):
        # A low-speed bit is 22 SWIM clocks long.
        # There are options to shorten bits to 10 clocks or use HSI rather
        # than HSI/2 as the SWIM clock but the longest valid bit should be no
        # more than this many samples. This does not need to be accurate.
        # It exists simply to prevent bits extending unecessarily far into
        # trailing bus-idle periods. This will be adjusted every time we see
        # a synchronization frame or start bit in order to show idle periods
        # as accurately as possible.
        self.bit_reflen = math.ceil(self.samplerate * 22 / self.swim_clock)

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)

        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        # A synchronization frame is a low that lasts for more than 64 but no
        # more than 128 SWIM clock periods based on the standard SWIM clock.
        # Note: we also allow for the possibility that the SWIM clock divisor
        # has been disabled here.
        self.sync_reflen_min = math.floor(self.samplerate * 64 / self.HSI_max)
        self.sync_reflen_max = math.ceil(self.samplerate * 128 / (self.HSI_min / 2))

        self.debug = True if self.options['debug'] == 'yes' else False

        # The SWIM entry sequence is 4 pulses at 2kHz followed by 4 at 1kHz.
        self.eseq_reflen = math.ceil(self.samplerate / 2048)

        self.adjust_timings()

    def protocol(self):
        if self.proto_state == 'CMD':
            # Command
            if self.bitseq_value == 0x00:
                self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [10, ['system reset', 'SRST', '!']])
            elif self.bitseq_value == 0x01:
                self.proto_state = 'N'
                self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [10, ['read on-the-fly', 'ROTF', 'r']])
            elif self.bitseq_value == 0x02:
                self.proto_state = 'N'
                self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [10, ['write on-the-fly', 'WOTF', 'w']])
            else:
                self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [9, ['unknown', 'UNK']])
        elif self.proto_state == 'N':
            # Number of bytes
            self.proto_byte_count = self.bitseq_value
            self.proto_state = '@E'
            self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [11, ['byte count 0x%02x' % self.bitseq_value, 'bytes 0x%02x' % self.bitseq_value, '0x%02x' % self.bitseq_value, '%02x' % self.bitseq_value, '%x' % self.bitseq_value]])
        elif self.proto_state == '@E':
            # Address byte 1
            self.proto_addr = self.bitseq_value
            self.proto_addr_start = self.bitseq_start
            self.proto_state = '@H'
        elif self.proto_state == '@H':
            # Address byte 2
            self.proto_addr = (self.proto_addr << 8) | self.bitseq_value
            self.proto_state = '@L'
        elif self.proto_state == '@L':
            # Address byte 3
            self.proto_addr = (self.proto_addr << 8) | self.bitseq_value
            self.proto_state = 'D'
            self.put(self.proto_addr_start, self.bitseq_end, self.out_ann, [12, ['address 0x%06x' % self.proto_addr, 'addr 0x%06x' % self.proto_addr, '0x%06x' % self.proto_addr, '%06x' %self.proto_addr, '%x' % self.proto_addr]])
        else:
            if self.proto_byte_count > 0:
                self.proto_byte_count -= 1
                if self.proto_byte_count == 0:
                    self.proto_state = 'CMD'

            self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [13 + self.bitseq_dir, ['0x%02x' % self.bitseq_value, '%02x' % self.bitseq_value, '%x' % self.bitseq_value]])
            self.put(self.bitseq_start, self.bitseq_end, self.out_binary, [0 + self.bitseq_dir, bytes([self.bitseq_value])])
            if self.debug:
                self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [15, ['%d more' % self.proto_byte_count, '%d' % self.proto_byte_count]])

    def bitseq(self, bitstart, bitend, bit):
        if self.bitseq_len == 0:
            # Looking for start of a bit sequence (command or byte).
            self.bit_reflen = bitend - bitstart
            self.bitseq_value = 0
            self.bitseq_dir = bit
            self.bitseq_len = 1
            self.put(bitstart, bitend, self.out_ann, [2 + self.bitseq_dir, ['start', 's']])
        elif (self.proto_state == 'CMD' and self.bitseq_len == 4) or (self.proto_state != 'CMD' and self.bitseq_len == 9):
            # Parity bit
            self.bitseq_end = bitstart
            self.bitseq_len += 1

            self.put(bitstart, bitend, self.out_ann, [4, ['parity', 'par', 'p']])

            # The start bit is not data but was used for parity calculation.
            self.bitseq_value &= 0xff
            self.put(self.bitseq_start, self.bitseq_end, self.out_ann, [7 + self.bitseq_dir, ['0x%02x' % self.bitseq_value, '%02x' % self.bitseq_value, '%x' % self.bitseq_value]])
        elif (self.proto_state == 'CMD' and self.bitseq_len == 5) or (self.proto_state != 'CMD' and self.bitseq_len == 10):
            # ACK/NACK bit.
            if bit:
                self.put(bitstart, bitend, self.out_ann, [5, ['ack', 'a']])
            else:
                self.put(bitstart, bitend, self.out_ann, [6, ['nack', 'n']])

            # We only pass data that was ack'd up the stack.
            if bit:
                self.protocol()

            self.bitseq_len = 0
        else:
            if self.bitseq_len == 1:
                self.bitseq_start = bitstart
            self.bitseq_value = (self.bitseq_value << 1) | bit
            self.bitseq_len += 1

    def bit(self, start, mid, end):
        if mid - start >= end - mid:
            self.put(start, end, self.out_ann, [0, ['0']])
            bit = 0
        else:
            self.put(start, end, self.out_ann, [0, ['1']])
            bit = 1

        self.bitseq(start, end, bit)

    def detect_synchronize_frame(self, start, end):
        # Strictly speaking, synchronization frames are only recognised when
        # SWIM is active. A falling edge on reset disables SWIM and an enter
        # sequence is needed to re-enable it. However we do not want to be
        # reliant on seeing the NRST pin just for that and we also want to be
        # able to decode SWIM even if we just sample parts of the dialogue.
        # For this reason we limit ourselves to only recognizing
        # synchronization frames that have believable lengths based on our
        # knowledge of the range of possible SWIM clocks.
        if self.samplenum - self.eseq_edge[1][1] >= self.sync_reflen_min and self.samplenum - self.eseq_edge[1][1] <= self.sync_reflen_max:
            self.put(self.eseq_edge[1][1], self.samplenum, self.out_ann, [1, ['synchronization frame', 'synchronization', 'sync', 's']])

            # A low that lasts for more than 64 SWIM clock periods causes a
            # reset of the SWIM communication state machine and will switch
            # the SWIM to low-speed mode (SWIM_CSR.HS is cleared).
            self.reset()

            # The low SHOULD last 128 SWIM clocks. This is used to
            # resynchronize in order to allow for variation in the frequency
            # of the internal RC oscillator.
            self.swim_clock = 128 * (self.samplerate / (self.samplenum - self.eseq_edge[1][1]))
            self.adjust_timings()

    def eseq_potential_start(self, start, end):
        self.eseq_pairstart = start
        self.eseq_reflen = end - start
        self.eseq_pairnum = 1

    def detect_enter_sequence(self, start, end):
        # According to the spec the enter sequence is four pulses at 2kHz
        # followed by four at 1kHz. We do not check the frequency but simply
        # check the lengths of successive pulses against the first. This means
        # we have no need to account for the accuracy (or lack of) of the
        # host's oscillator.
        if self.eseq_pairnum == 0 or abs(self.eseq_reflen - (end - start)) > 2:
            self.eseq_potential_start(start, end)

        elif self.eseq_pairnum < 4:
            # The next three pulses should be the same length as the first.
            self.eseq_pairnum += 1

            if self.eseq_pairnum == 4:
                self.eseq_reflen /= 2
        else:
            # The final four pulses should each be half the length of the
            # initial pair. Again, a mismatch causes us to reset and use the
            # current pulse as a new potential enter sequence start.
            self.eseq_pairnum += 1
            if self.eseq_pairnum == 8:
                # Four matching pulses followed by four more that match each
                # other but are half the length of the first 4. SWIM is active!
                self.put(self.eseq_pairstart, end, self.out_ann, [1, ['enter sequence', 'enter seq', 'enter', 'ent', 'e']])
                self.eseq_pairnum = 0

    def decode(self):
        while True:
            if self.bit_maxlen >= 0:
                (swim,) = self.wait()
                self.bit_maxlen -= 1
            else:
                (swim,) = self.wait({0: 'e'})

            if swim != self.eseq_edge[1][0]:
                if swim == 1 and self.eseq_edge[1][1] is not None:
                    self.detect_synchronize_frame(self.eseq_edge[1][1], self.samplenum)
                    if self.eseq_edge[0][1] is not None:
                        self.detect_enter_sequence(self.eseq_edge[0][1], self.samplenum)
                self.eseq_edge.pop(0)
                self.eseq_edge.append([swim, self.samplenum])

            if (swim != self.bit_edge[1][0] and (swim != 1 or self.bit_edge[1][0] != -1)) or self.bit_maxlen == 0:
                if self.bit_maxlen == 0 and self.bit_edge[1][0] == 1:
                    swim = -1

                if self.bit_edge[1][0] != 0 and swim == 0:
                    self.bit_maxlen = self.bit_reflen

                if self.bit_edge[0][0] == 0 and self.bit_edge[1][0] == 1 and self.samplenum - self.bit_edge[0][1] <= self.bit_reflen + 2:
                    self.bit(self.bit_edge[0][1], self.bit_edge[1][1], self.samplenum)

                self.bit_edge.pop(0)
                self.bit_edge.append([swim, self.samplenum])



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Steve R <steversig@virginmedia.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
OOK decodes On-off keying based remote control protocols.

It is aimed at 433MHz but should also work with other common RC frequencies.
The input can be captured directly from a transmitter (before the modulation
stage) or demodulated by an RF receiver.

Over the air captured traces will be a lot noisier and will probably need the
area of interest to be zoomed onto, then selected with the "Cursors" and the
"Save Selected Range As" feature to be used to extract it from the noise.

There is a limited amount of pre-filtering and garbage removal built into the
decoder which can sometimes extract signals directly from a larger over the air
trace. It depends heavily on your environment.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012-2014 Uwe Hermann <uwe@hermann-uwe.de>
## Copyright (C) 2013 Matt Ranostay <mranostay@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import re
import sigrokdecode as srd
from common.srdhelper import bcd2int

days_of_week = (
    'Sunday', 'Monday', 'Tuesday', 'Wednesday',
    'Thursday', 'Friday', 'Saturday',
)

regs = (
    'Seconds', 'Minutes', 'Hours', 'Day', 'Date', 'Month', 'Year',
    'Control', 'RAM',
)

bits = (
    'Clock halt', 'Seconds', 'Reserved', 'Minutes', '12/24 hours', 'AM/PM',
    'Hours', 'Day', 'Date', 'Month', 'Year', 'OUT', 'SQWE', 'RS', 'RAM',
)

rates = {
    0b00: '1Hz',
    0b01: '4096Hz',
    0b10: '8192Hz',
    0b11: '32768Hz',
}

DS1307_I2C_ADDRESS = 0x68

def regs_and_bits():
    l = [('reg-' + r.lower(), r + ' register') for r in regs]
    l += [('bit-' + re.sub('\/| ', '-', b).lower(), b + ' bit') for b in bits]
    return tuple(l)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ds1307'
    name = 'DS1307'
    longname = 'Dallas DS1307'
    desc = 'Dallas DS1307 realtime clock module protocol.'
    license = 'gplv2+'
    inputs = ['i2c']
    outputs = []
    tags = ['Clock/timing', 'IC']
    annotations =  regs_and_bits() + (
        ('read-datetime', 'Read date/time'),
        ('write-datetime', 'Write date/time'),
        ('reg-read', 'Register read'),
        ('reg-write', 'Register write'),
        ('warnings', 'Warnings'),
    )
    annotation_rows = (
        ('bits', 'Bits', tuple(range(9, 24))),
        ('regs', 'Registers', tuple(range(9))),
        ('date-time', 'Date/time', (24, 25, 26, 27)),
        ('warnings', 'Warnings', (28,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.hours = -1
        self.minutes = -1
        self.seconds = -1
        self.days = -1
        self.date = -1
        self.months = -1
        self.years = -1
        self.bits = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def putd(self, bit1, bit2, data):
        self.put(self.bits[bit1][1], self.bits[bit2][2], self.out_ann, data)

    def putr(self, bit):
        self.put(self.bits[bit][1], self.bits[bit][2], self.out_ann,
                 [11, ['Reserved bit', 'Reserved', 'Rsvd', 'R']])

    def handle_reg_0x00(self, b): # Seconds (0-59) / Clock halt bit
        self.putd(7, 0, [0, ['Seconds', 'Sec', 'S']])
        ch = 1 if (b & (1 << 7)) else 0
        self.putd(7, 7, [9, ['Clock halt: %d' % ch, 'Clk hlt: %d' % ch,
                        'CH: %d' % ch, 'CH']])
        s = self.seconds = bcd2int(b & 0x7f)
        self.putd(6, 0, [10, ['Second: %d' % s, 'Sec: %d' % s, 'S: %d' % s, 'S']])

    def handle_reg_0x01(self, b): # Minutes (0-59)
        self.putd(7, 0, [1, ['Minutes', 'Min', 'M']])
        self.putr(7)
        m = self.minutes = bcd2int(b & 0x7f)
        self.putd(6, 0, [12, ['Minute: %d' % m, 'Min: %d' % m, 'M: %d' % m, 'M']])

    def handle_reg_0x02(self, b): # Hours (1-12+AM/PM or 0-23)
        self.putd(7, 0, [2, ['Hours', 'H']])
        self.putr(7)
        ampm_mode = True if (b & (1 << 6)) else False
        if ampm_mode:
            self.putd(6, 6, [13, ['12-hour mode', '12h mode', '12h']])
            a = 'PM' if (b & (1 << 5)) else 'AM'
            self.putd(5, 5, [14, [a, a[0]]])
            h = self.hours = bcd2int(b & 0x1f)
            self.putd(4, 0, [15, ['Hour: %d' % h, 'H: %d' % h, 'H']])
        else:
            self.putd(6, 6, [13, ['24-hour mode', '24h mode', '24h']])
            h = self.hours = bcd2int(b & 0x3f)
            self.putd(5, 0, [15, ['Hour: %d' % h, 'H: %d' % h, 'H']])

    def handle_reg_0x03(self, b): # Day / day of week (1-7)
        self.putd(7, 0, [3, ['Day of week', 'Day', 'D']])
        for i in (7, 6, 5, 4, 3):
            self.putr(i)
        w = self.days = bcd2int(b & 0x07)
        ws = days_of_week[self.days - 1]
        self.putd(2, 0, [16, ['Weekday: %s' % ws, 'WD: %s' % ws, 'WD', 'W']])

    def handle_reg_0x04(self, b): # Date (1-31)
        self.putd(7, 0, [4, ['Date', 'D']])
        for i in (7, 6):
            self.putr(i)
        d = self.date = bcd2int(b & 0x3f)
        self.putd(5, 0, [17, ['Date: %d' % d, 'D: %d' % d, 'D']])

    def handle_reg_0x05(self, b): # Month (1-12)
        self.putd(7, 0, [5, ['Month', 'Mon', 'M']])
        for i in (7, 6, 5):
            self.putr(i)
        m = self.months = bcd2int(b & 0x1f)
        self.putd(4, 0, [18, ['Month: %d' % m, 'Mon: %d' % m, 'M: %d' % m, 'M']])

    def handle_reg_0x06(self, b): # Year (0-99)
        self.putd(7, 0, [6, ['Year', 'Y']])
        y = self.years = bcd2int(b & 0xff)
        self.years += 2000
        self.putd(7, 0, [19, ['Year: %d' % y, 'Y: %d' % y, 'Y']])

    def handle_reg_0x07(self, b): # Control Register
        self.putd(7, 0, [7, ['Control', 'Ctrl', 'C']])
        for i in (6, 5, 3, 2):
            self.putr(i)
        o = 1 if (b & (1 << 7)) else 0
        s = 1 if (b & (1 << 4)) else 0
        s2 = 'en' if (b & (1 << 4)) else 'dis'
        r = rates[b & 0x03]
        self.putd(7, 7, [20, ['Output control: %d' % o,
            'OUT: %d' % o, 'O: %d' % o, 'O']])
        self.putd(4, 4, [21, ['Square wave output: %sabled' % s2,
            'SQWE: %sabled' % s2, 'SQWE: %d' % s, 'S: %d' % s, 'S']])
        self.putd(1, 0, [22, ['Square wave output rate: %s' % r,
            'Square wave rate: %s' % r, 'SQW rate: %s' % r, 'Rate: %s' % r,
            'RS: %s' % s, 'RS', 'R']])

    def handle_reg_0x3f(self, b): # RAM (bytes 0x08-0x3f)
        self.putd(7, 0, [8, ['RAM', 'R']])
        self.putd(7, 0, [23, ['SRAM: 0x%02X' % b, '0x%02X' % b]])

    def output_datetime(self, cls, rw):
        # TODO: Handle read/write of only parts of these items.
        d = '%s, %02d.%02d.%4d %02d:%02d:%02d' % (
            days_of_week[self.days - 1], self.date, self.months,
            self.years, self.hours, self.minutes, self.seconds)
        self.put(self.ss_block, self.es, self.out_ann,
                 [cls, ['%s date/time: %s' % (rw, d)]])

    def handle_reg(self, b):
        r = self.reg if self.reg < 8 else 0x3f
        fn = getattr(self, 'handle_reg_0x%02x' % r)
        fn(b)
        # Honor address auto-increment feature of the DS1307. When the
        # address reaches 0x3f, it will wrap around to address 0.
        self.reg += 1
        if self.reg > 0x3f:
            self.reg = 0

    def is_correct_chip(self, addr):
        if addr == DS1307_I2C_ADDRESS:
            return True
        self.put(self.ss_block, self.es, self.out_ann,
                 [28, ['Ignoring non-DS1307 data (slave 0x%02X)' % addr]])
        return False

    def decode(self, ss, es, data):
        cmd, databyte = data

        # Collect the 'BITS' packet, then return. The next packet is
        # guaranteed to belong to these bits we just stored.
        if cmd == 'BITS':
            self.bits = databyte
            return

        # Store the start/end samples of this I²C packet.
        self.ss, self.es = ss, es

        # State machine.
        if self.state == 'IDLE':
            # Wait for an I²C START condition.
            if cmd != 'START':
                return
            self.state = 'GET SLAVE ADDR'
            self.ss_block = ss
        elif self.state == 'GET SLAVE ADDR':
            # Wait for an address write operation.
            if cmd != 'ADDRESS WRITE':
                return
            if not self.is_correct_chip(databyte):
                self.state = 'IDLE'
                return
            self.state = 'GET REG ADDR'
        elif self.state == 'GET REG ADDR':
            # Wait for a data write (master selects the slave register).
            if cmd != 'DATA WRITE':
                return
            self.reg = databyte
            self.state = 'WRITE RTC REGS'
        elif self.state == 'WRITE RTC REGS':
            # If we see a Repeated Start here, it's an RTC read.
            if cmd == 'START REPEAT':
                self.state = 'READ RTC REGS'
                return
            # Otherwise: Get data bytes until a STOP condition occurs.
            if cmd == 'DATA WRITE':
                self.handle_reg(databyte)
            elif cmd == 'STOP':
                self.output_datetime(25, 'Written')
                self.state = 'IDLE'
        elif self.state == 'READ RTC REGS':
            # Wait for an address read operation.
            if cmd != 'ADDRESS READ':
                return
            if not self.is_correct_chip(databyte):
                self.state = 'IDLE'
                return
            self.state = 'READ RTC REGS2'
        elif self.state == 'READ RTC REGS2':
            if cmd == 'DATA READ':
                self.handle_reg(databyte)
            elif cmd == 'STOP':
                self.output_datetime(24, 'Read')
                self.state = 'IDLE'



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Bart de Waal <bart@waalamo.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from math import ceil

RX = 0
TX = 1
rxtx_channels = ('RX', 'TX')

class No_more_data(Exception):
    '''This exception is a signal that we should stop parsing an ADU as there
    is no more data to parse.'''
    pass

class Data:
    '''The Data class is used to hold the bytes from the serial decode.'''
    def __init__(self, start, end, data):
        self.start = start
        self.end = end
        self.data = data

class Modbus_ADU:
    '''An Application Data Unit is what Modbus calls one message.
    Protocol decoders are supposed to keep track of state and then provide
    decoded data to the backend as it reads it. In Modbus' case, the state is
    the ADU up to that point. This class represents the state and writes the
    messages to the backend.
    This class is for the common infrastructure between CS and SC. It should
    not be used directly, only inhereted from.'''

    def __init__(self, parent, start, write_channel, annotation_prefix):
        self.data = [] # List of all the data received up to now
        self.parent = parent # Reference to the decoder object
        self.start = start
        self.last_read = start # The last moment parsed by this ADU object
        self.write_channel = write_channel
        self.last_byte_put = -1
        self.annotation_prefix = annotation_prefix
        # Any Modbus message needs to be at least 4 bytes long. The Modbus
        # function may make this longer.
        self.minimum_length = 4

        # This variable is used by an external function to determine when the
        # next frame should be started.
        self.startNewFrame = False

        # If there is an error in a frame, we'd like to highlight it. Keep
        # track of errors.
        self.hasError = False

    def add_data(self, start, end, data):
        '''Let the frame handle another piece of data.
        start: start of this data
        end: end of this data
        data: data as received from the UART decoder'''
        ptype, rxtx, pdata = data
        self.last_read = end
        if ptype == 'DATA':
            self.data.append(Data(start, end, pdata[0]))
            self.parse() # parse() is defined in the specific type of ADU.

    def puti(self, byte_to_put, annotation, message):
        '''This class keeps track of how much of the data has already been
        annotated. This function tells the parent class to write message, but
        only if it hasn't written about this bit before.
        byte_to_put: Only write if it hasn't yet written byte_to_put. It will
                     write from the start of self.last_byte_put+1 to the end
                     of byte_to_put.
        annotation: Annotation to write to, without annotation_prefix.
        message: Message to write.'''
        if byte_to_put > len(self.data) - 1:
            # If the byte_to_put hasn't been read yet.
            raise No_more_data

        if annotation == 'error':
            self.hasError = True

        if byte_to_put > self.last_byte_put:
            self.parent.puta(
                self.data[self.last_byte_put + 1].start,
                self.data[byte_to_put].end,
                self.annotation_prefix + annotation,
                message)
            self.last_byte_put = byte_to_put
            raise No_more_data

    def putl(self, annotation, message, maximum=None):
        '''Puts the last byte on the stack with message. The contents of the
        last byte will be applied to message using format.'''
        last_byte_address = len(self.data) - 1
        if maximum is not None and last_byte_address > maximum:
            return
        self.puti(last_byte_address, annotation,
                  message.format(self.data[-1].data))

    def close(self, message_overflow):
        '''Function to be called when next message is started. As there is
        always space between one message and the next, we can use that space
        for errors at the end.'''
        # TODO: Figure out how to make this happen for last message.
        data = self.data
        if len(data) < self.minimum_length:
            if len(data) == 0:
                # Sometimes happens with noise, safe to ignore.
                return
            self.parent.puta(
                data[self.last_byte_put].end, message_overflow,
                self.annotation_prefix + 'error',
                'Message too short or not finished')
            self.hasError = True
        if self.hasError and self.parent.options['scchannel'] != self.parent.options['cschannel']:
            # If we are decoding different channels (so client->server and
            # server->client messages can be separated) we like to mark blocks
            # containing errors. We don't do this when decoding the same
            # channel as both a client->server and server->client frame, and
            # one of those is bound to contain an error, making highlighting
            # frames useless.
            self.parent.puta(data[0].start, data[-1].end,
                             'error-indication', 'Frame contains error')
        if len(data) > 256:
            try:
                self.puti(len(data) - 1, 'error',
                    'Modbus data frames are limited to 256 bytes')
            except No_more_data:
                pass

    def check_crc(self, byte_to_put):
        '''Check the CRC code, data[byte_to_put] is the 2nd byte of the CRC.'''
        crc_byte1, crc_byte2 = self.calc_crc(byte_to_put)
        data = self.data
        if data[-2].data == crc_byte1 and data[-1].data == crc_byte2:
            self.puti(byte_to_put, 'crc', 'CRC correct')
        else:
            self.puti(byte_to_put, 'error',
                'CRC should be {} {}'.format(crc_byte1, crc_byte2))

    def half_word(self, start):
        '''Return the half word (16 bit) value starting at start bytes in. If
        it goes out of range it raises the usual errors.'''
        if (start + 1) > (len(self.data) - 1):
            # If there isn't enough length to access data[start + 1].
            raise No_more_data
        return self.data[start].data * 0x100 + self.data[start + 1].data

    def calc_crc(self, last_byte):
        '''Calculate the CRC, as described in the spec.
        The last byte of the CRC should be data[last_byte].'''
        if last_byte < 3:
            # Every Modbus ADU should be as least 4 long, so we should never
            # have to calculate a CRC on something shorter.
            raise Exception('Could not calculate CRC: message too short')

        result = 0xFFFF
        magic_number = 0xA001 # As defined in the modbus specification.
        for byte in self.data[:last_byte - 1]:
            result = result ^ byte.data
            for i in range(8):
                LSB = result & 1
                result = result >> 1
                if (LSB): # If the LSB is true.
                    result = result ^ magic_number
        byte1 = result & 0xFF
        byte2 = (result & 0xFF00) >> 8
        return (byte1, byte2)

    def parse_write_single_coil(self):
        '''Parse function 5, write single coil.'''
        self.minimum_length = 8

        self.puti(1, 'function', 'Function 5: Write Single Coil')

        address = self.half_word(2)
        self.puti(3, 'address',
            'Address 0x{:X} / {:d}'.format(address, address + 10000))

        raw_value = self.half_word(4)
        value = 'Invalid Coil Value'
        if raw_value == 0x0000:
            value = 'Coil Value OFF'
        elif raw_value == 0xFF00:
            value = 'Coil Value ON'
        self.puti(5, 'data', value)

        self.check_crc(7)

    def parse_write_single_register(self):
        '''Parse function 6, write single register.'''
        self.minimum_length = 8

        self.puti(1, 'function', 'Function 6: Write Single Register')

        address = self.half_word(2)
        self.puti(3, 'address',
            'Address 0x{:X} / {:d}'.format(address, address + 30000))

        value = self.half_word(4)
        value_formatted = 'Register Value 0x{0:X} / {0:d}'.format(value)
        self.puti(5, 'data', value_formatted)

        self.check_crc(7)

    def parse_diagnostics(self):
        '''Parse function 8, diagnostics. This function has many subfunctions,
        but they are all more or less the same.'''
        self.minimum_length = 8

        self.puti(1, 'function', 'Function 8: Diagnostics')

        diag_subfunction = {
            0: 'Return Query data',
            1: 'Restart Communications Option',
            2: 'Return Diagnostics Register',
            3: 'Change ASCII Input Delimiter',
            4: 'Force Listen Only Mode',
            10: 'Clear Counters and Diagnostic Register',
            11: 'Return Bus Message Count',
            12: 'Return Bus Communication Error Count',
            13: 'Return Bus Exception Error Count',
            14: 'Return Slave Message Count',
            15: 'Return Slave No Response Count',
            16: 'Return Slave NAK Count',
            17: 'Return Slave Busy Count',
            18: 'Return Bus Character Overrun Count',
            20: 'Return Overrun Counter and Flag',
        }
        subfunction = self.half_word(2)
        subfunction_name = diag_subfunction.get(subfunction,
                                                'Reserved subfunction')
        self.puti(3, 'data',
            'Subfunction {}: {}'.format(subfunction, subfunction_name))

        diagnostic_data = self.half_word(4)
        self.puti(5, 'data',
            'Data Field: {0} / 0x{0:04X}'.format(diagnostic_data))

        self.check_crc(7)

    def parse_mask_write_register(self):
        '''Parse function 22, Mask Write Register.'''
        self.minimum_length = 10
        data = self.data

        self.puti(1, 'function', 'Function 22: Mask Write Register')

        address = self.half_word(2)
        self.puti(3, 'address',
            'Address 0x{:X} / {:d}'.format(address, address + 30001))

        self.half_word(4) # To make sure we don't oveflow data.
        and_mask_1 = data[4].data
        and_mask_2 = data[5].data
        self.puti(5, 'data',
            'AND mask: {:08b} {:08b}'.format(and_mask_1, and_mask_2))

        self.half_word(6) # To make sure we don't oveflow data.
        or_mask_1 = data[6].data
        or_mask_2 = data[7].data
        self.puti(7, 'data',
            'OR mask: {:08b} {:08b}'.format(or_mask_1, or_mask_2))

        self.check_crc(9)

    def parse_not_implemented(self):
        '''Explicitly mark certain functions as legal functions, but not
        implemented in this parser. This is due to the author not being able to
        find anything (hardware or software) that supports these functions.'''
        # TODO: Implement these functions.

        # Mentioning what function it is is no problem.
        function = self.data[1].data
        functionname = {
            20: 'Read File Record',
            21: 'Write File Record',
            24: 'Read FIFO Queue',
            43: 'Read Device Identification/Encapsulated Interface Transport',
        }[function]
        self.puti(1, 'function',
            'Function {}: {} (not supported)'.format(function, functionname))

        # From there on out we can keep marking it unsupported.
        self.putl('data', 'This function is not currently supported')

class Modbus_ADU_SC(Modbus_ADU):
    '''SC stands for Server -> Client.'''
    def parse(self):
        '''Select which specific Modbus function we should parse.'''
        data = self.data

        # This try-catch is being used as flow control.
        try:
            server_id = data[0].data
            if 1 <= server_id <= 247:
                message = 'Slave ID: {}'.format(server_id)
            else:
                message = 'Slave ID {} is invalid'
            self.puti(0, 'server-id', message)

            function = data[1].data
            if function == 1 or function == 2:
                self.parse_read_bits()
            elif function == 3 or function == 4 or function == 23:
                self.parse_read_registers()
            elif function == 5:
                self.parse_write_single_coil()
            elif function == 6:
                self.parse_write_single_register()
            elif function == 7:
                self.parse_read_exception_status()
            elif function == 8:
                self.parse_diagnostics()
            elif function == 11:
                self.parse_get_comm_event_counter()
            elif function == 12:
                self.parse_get_comm_event_log()
            elif function == 15 or function == 16:
                self.parse_write_multiple()
            elif function == 17:
                self.parse_report_server_id()
            elif function == 22:
                self.parse_mask_write_register()
            elif function in {21, 21, 24, 43}:
                self.parse_not_implemented()
            elif function > 0x80:
                self.parse_error()
            else:
                self.puti(1, 'error',
                          'Unknown function: {}'.format(data[1].data))
                self.putl('error', 'Unknown function')

            # If the message gets here without raising an exception, the
            # message goes on longer than it should.
            self.putl('error', 'Message too long')

        except No_more_data:
            # Just a message saying we don't need to parse anymore this round.
            pass

    def parse_read_bits(self):
        self.mimumum_length = 5

        data = self.data
        function = data[1].data

        if function == 1:
            self.puti(1, 'function', 'Function 1: Read Coils')
        else:
            self.puti(1, 'function', 'Function 2: Read Discrete Inputs')

        bytecount = self.data[2].data
        self.minimum_length = 5 + bytecount # 3 before data, 2 CRC.
        self.puti(2, 'length', 'Byte count: {}'.format(bytecount))

        # From here on out, we expect registers on 3 and 4, 5 and 6 etc.
        # So registers never start when the length is even.
        self.putl('data', '{:08b}', bytecount + 2)
        self.check_crc(bytecount + 4)

    def parse_read_registers(self):
        self.mimumum_length = 5

        data = self.data

        function = data[1].data
        if function == 3:
            self.puti(1, 'function', 'Function 3: Read Holding Registers')
        elif function == 4:
            self.puti(1, 'function', 'Function 4: Read Input Registers')
        elif function == 23:
            self.puti(1, 'function', 'Function 23: Read/Write Multiple Registers')

        bytecount = self.data[2].data
        self.minimum_length = 5 + bytecount # 3 before data, 2 CRC.
        if bytecount % 2 == 0:
            self.puti(2, 'length', 'Byte count: {}'.format(bytecount))
        else:
            self.puti(2, 'error',
                'Error: Odd byte count ({})'.format(bytecount))

        # From here on out, we expect registers on 3 and 4, 5 and 6 etc.
        # So registers never start when the length is even.
        if len(data) % 2 == 1:
            register_value = self.half_word(-2)
            self.putl('data', '0x{0:04X} / {0}'.format(register_value),
                      bytecount + 2)
        else:
            raise No_more_data

        self.check_crc(bytecount + 4)

    def parse_read_exception_status(self):
        self.mimumum_length = 5

        self.puti(1, 'function', 'Function 7: Read Exception Status')
        exception_status = self.data[2].data
        self.puti(2, 'data',
                  'Exception status: {:08b}'.format(exception_status))
        self.check_crc(4)

    def parse_get_comm_event_counter(self):
        self.mimumum_length = 8

        self.puti(1, 'function', 'Function 11: Get Comm Event Counter')

        status = self.half_word(2)
        if status == 0x0000:
            self.puti(3, 'data', 'Status: not busy')
        elif status == 0xFFFF:
            self.puti(3, 'data', 'Status: busy')
        else:
            self.puti(3, 'error', 'Bad status: 0x{:04X}'.format(status))

        count = self.half_word(4)
        self.puti(5, 'data', 'Event Count: {}'.format(count))
        self.check_crc(7)

    def parse_get_comm_event_log(self):
        self.mimumum_length = 11
        self.puti(1, 'function', 'Function 12: Get Comm Event Log')

        data = self.data

        bytecount = data[2].data
        self.puti(2, 'length', 'Bytecount: {}'.format(bytecount))
        # The bytecount is the length of everything except the slaveID,
        # function code, bytecount and CRC.
        self.mimumum_length = 5 + bytecount

        status = self.half_word(3)
        if status == 0x0000:
            self.puti(4, 'data', 'Status: not busy')
        elif status == 0xFFFF:
            self.puti(4, 'data', 'Status: busy')
        else:
            self.puti(4, 'error', 'Bad status: 0x{:04X}'.format(status))

        event_count = self.half_word(5)
        self.puti(6, 'data', 'Event Count: {}'.format(event_count))

        message_count = self.half_word(7)
        self.puti(8, 'data', 'Message Count: {}'.format(message_count))

        self.putl('data', 'Event: 0x{:02X}'.format(data[-1].data),
                  bytecount + 2)

        self.check_crc(bytecount + 4)

    def parse_write_multiple(self):
        '''Function 15 and 16 are almost the same, so we can parse them both
        using one function.'''
        self.mimumum_length = 8

        function = self.data[1].data
        if function == 15:
            data_unit = 'Coils'
            max_outputs = 0x07B0
            long_address_offset = 10001
        elif function == 16:
            data_unit = 'Registers'
            max_outputs = 0x007B
            long_address_offset = 30001

        self.puti(1, 'function',
            'Function {}: Write Multiple {}'.format(function, data_unit))

        starting_address = self.half_word(2)
        # Some instruction manuals use a long form name for addresses, this is
        # listed here for convienience.
        address_name = long_address_offset + starting_address
        self.puti(3, 'address',
            'Start at address 0x{:X} / {:d}'.format(starting_address,
                                                    address_name))

        quantity_of_outputs = self.half_word(4)
        if quantity_of_outputs <= max_outputs:
            self.puti(5, 'data',
                'Write {} {}'.format(quantity_of_outputs, data_unit))
        else:
            self.puti(5, 'error',
                'Bad value: {} {}. Max is {}'.format(quantity_of_outputs,
                                                     data_unit, max_outputs))

        self.check_crc(7)

    def parse_report_server_id(self):
        # Buildup of this function:
        # 1 byte serverID
        # 1 byte function (17)
        # 1 byte bytecount
        # 1 byte serverID (counts for bytecount)
        # 1 byte Run Indicator Status (counts for bytecount)
        # bytecount - 2 bytes of device specific data (counts for bytecount)
        # 2 bytes of CRC
        self.mimumum_length = 7
        data = self.data
        self.puti(1, 'function', 'Function 17: Report Server ID')

        bytecount = data[2].data
        self.puti(2, 'length', 'Data is {} bytes long'.format(bytecount))

        self.puti(3, 'data', 'serverID: {}'.format(data[3].data))

        run_indicator_status = data[4].data
        if run_indicator_status == 0x00:
            self.puti(4, 'data', 'Run Indicator status: Off')
        elif run_indicator_status == 0xFF:
            self.puti(4, 'data', 'Run Indicator status: On')
        else:
            self.puti(4, 'error',
                'Bad Run Indicator status: 0x{:X}'.format(run_indicator_status))

        self.putl('data', 'Device specific data: {}, "{}"'.format(data[-1].data,
                  chr(data[-1].data)), 2 + bytecount)

        self.check_crc(4 + bytecount)

    def parse_error(self):
        '''Parse a Modbus error message.'''
        self.mimumum_length = 5
        # The function code of an error is always 0x80 above the function call
        # that caused it.
        functioncode = self.data[1].data - 0x80

        functions = {
            1: 'Read Coils',
            2: 'Read Discrete Inputs',
            3: 'Read Holding Registers',
            4: 'Read Input Registers',
            5: 'Write Single Coil',
            6: 'Write Single Register',
            7: 'Read Exception Status',
            8: 'Diagnostic',
            11: 'Get Com Event Counter',
            12: 'Get Com Event Log',
            15: 'Write Multiple Coils',
            16: 'Write Multiple Registers',
            17: 'Report Slave ID',
            20: 'Read File Record',
            21: 'Write File Record',
            22: 'Mask Write Register',
            23: 'Read/Write Multiple Registers',
            24: 'Read FIFO Queue',
            43: 'Read Device Identification/Encapsulated Interface Transport',
        }
        functionname = '{}: {}'.format(functioncode,
            functions.get(functioncode, 'Unknown function'))
        self.puti(1, 'function',
                  'Error for function {}'.format(functionname))

        error = self.data[2].data
        errorcodes = {
            1: 'Illegal Function',
            2: 'Illegal Data Address',
            3: 'Illegal Data Value',
            4: 'Slave Device Failure',
            5: 'Acknowledge',
            6: 'Slave Device Busy',
            8: 'Memory Parity Error',
            10: 'Gateway Path Unavailable',
            11: 'Gateway Target Device failed to respond',
        }
        errorname = '{}: {}'.format(error, errorcodes.get(error, 'Unknown'))
        self.puti(2, 'data', 'Error {}'.format(errorname))
        self.check_crc(4)

class Modbus_ADU_CS(Modbus_ADU):
    '''CS stands for Client -> Server.'''
    def parse(self):
        '''Select which specific Modbus function we should parse.'''
        data = self.data

        # This try-catch is being used as flow control.
        try:
            server_id = data[0].data
            message = ''
            if server_id == 0:
                message = 'Broadcast message'
            elif 1 <= server_id <= 247:
                message = 'Slave ID: {}'.format(server_id)
            elif 248 <= server_id <= 255:
                message = 'Slave ID: {} (reserved address)'.format(server_id)
            self.puti(0, 'server-id', message)

            function = data[1].data
            if function >= 1 and function <= 4:
                self.parse_read_data_command()
            if function == 5:
                self.parse_write_single_coil()
            if function == 6:
                self.parse_write_single_register()
            if function in {7, 11, 12, 17}:
                self.parse_single_byte_request()
            elif function == 8:
                self.parse_diagnostics()
            if function in {15, 16}:
                self.parse_write_multiple()
            elif function == 22:
                self.parse_mask_write_register()
            elif function == 23:
                self.parse_read_write_registers()
            elif function in {21, 21, 24, 43}:
                self.parse_not_implemented()
            else:
                self.puti(1, 'error',
                          'Unknown function: {}'.format(data[1].data))
                self.putl('error', 'Unknown function')

            # If the message gets here without raising an exception, the
            # message goes on longer than it should.
            self.putl('error', 'Message too long')

        except No_more_data:
            # Just a message saying we don't need to parse anymore this round.
            pass

    def parse_read_data_command(self):
        '''Interpret a command to read x units of data starting at address, ie
        functions 1, 2, 3 and 4, and write the result to the annotations.'''
        data = self.data
        self.minimum_length = 8

        function = data[1].data
        functionname = {1: 'Read Coils',
                        2: 'Read Discrete Inputs',
                        3: 'Read Holding Registers',
                        4: 'Read Input Registers',
                        }[function]

        self.puti(1, 'function',
                  'Function {}: {}'.format(function, functionname))

        starting_address = self.half_word(2)
        # Some instruction manuals use a long form name for addresses, this is
        # listed here for convienience.
        # Example: holding register 60 becomes 30061.
        address_name = 10000 * function + 1 + starting_address
        self.puti(3, 'address',
            'Start at address 0x{:X} / {:d}'.format(starting_address,
                                                    address_name))

        self.puti(5, 'length',
                  'Read {:d} units of data'.format(self.half_word(4)))
        self.check_crc(7)

    def parse_single_byte_request(self):
        '''Some Modbus functions have no arguments, this parses those.'''
        function = self.data[1].data
        function_name = {7: 'Read Exception Status',
                         11: 'Get Comm Event Counter',
                         12: 'Get Comm Event Log',
                         17: 'Report Slave ID',
                         }[function]
        self.puti(1, 'function',
                  'Function {}: {}'.format(function, function_name))

        self.check_crc(3)

    def parse_write_multiple(self):
        '''Function 15 and 16 are almost the same, so we can parse them both
        using one function.'''
        self.mimumum_length = 9

        function = self.data[1].data
        if function == 15:
            data_unit = 'Coils'
            max_outputs = 0x07B0
            ratio_bytes_data = 1/8
            long_address_offset = 10001
        elif function == 16:
            data_unit = 'Registers'
            max_outputs = 0x007B
            ratio_bytes_data = 2
            long_address_offset = 30001

        self.puti(1, 'function',
            'Function {}: Write Multiple {}'.format(function, data_unit))

        starting_address = self.half_word(2)
        # Some instruction manuals use a long form name for addresses, this is
        # listed here for convienience.
        address_name = long_address_offset + starting_address
        self.puti(3, 'address',
            'Start at address 0x{:X} / {:d}'.format(starting_address,
                                                    address_name))

        quantity_of_outputs = self.half_word(4)
        if quantity_of_outputs <= max_outputs:
            self.puti(5, 'length',
                'Write {} {}'.format(quantity_of_outputs, data_unit))
        else:
            self.puti(5, 'error',
                'Bad value: {} {}. Max is {}'.format(quantity_of_outputs,
                                                     data_unit, max_outputs))
        proper_bytecount = ceil(quantity_of_outputs * ratio_bytes_data)

        bytecount = self.data[6].data
        if bytecount == proper_bytecount:
            self.puti(6, 'length', 'Byte count: {}'.format(bytecount))
        else:
            self.puti(6, 'error',
                'Bad byte count, is {}, should be {}'.format(bytecount,
                                                             proper_bytecount))
        self.mimumum_length = bytecount + 9

        self.putl('data', 'Value 0x{:X}', 6 + bytecount)

        self.check_crc(bytecount + 8)

    def parse_read_file_record(self):
        self.puti(1, 'function', 'Function 20: Read file records')

        data = self.data

        bytecount = data[2].data

        self.minimum_length = 5 + bytecount
        # 1 for serverID, 1 for function, 1 for bytecount, 2 for CRC.

        if 0x07 <= bytecount <= 0xF5:
            self.puti(2, 'length', 'Request is {} bytes long'.format(bytecount))
        else:
            self.puti(2, 'error',
                'Request claims to be {} bytes long, legal values are between'
                ' 7 and 247'.format(bytecount))

        current_byte = len(data) - 1
        # Function 20 is a number of sub-requests, the first starting at 3,
        # the total length of the sub-requests is bytecount.
        if current_byte <= bytecount + 2:
            step = (current_byte - 3) % 7
            if step == 0:
                if data[current_byte].data == 6:
                    self.puti(current_byte, 'data', 'Start sub-request')
                else:
                    self.puti(current_byte, 'error',
                        'First byte of subrequest should be 0x06')
            elif step == 1:
                raise No_more_data
            elif step == 2:
                file_number = self.half_word(current_byte - 1)
                self.puti(current_byte, 'data',
                          'Read File number {}'.format(file_number))
            elif step == 3:
                raise No_more_data
            elif step == 4:
                record_number = self.half_word(current_byte - 1)
                self.puti(current_byte, 'address',
                    'Read from record number {}'.format(record_number))
                # TODO: Check if within range.
            elif step == 5:
                raise No_more_data
            elif step == 6:
                records_to_read = self.half_word(current_byte - 1)
                self.puti(current_byte, 'length',
                    'Read {} records'.format(records_to_read))
        self.check_crc(4 + bytecount)

    def parse_read_write_registers(self):
        '''Parse function 23: Read/Write multiple registers.'''
        self.minimum_length = 13

        self.puti(1, 'function', 'Function 23: Read/Write Multiple Registers')

        starting_address = self.half_word(2)
        # Some instruction manuals use a long form name for addresses, this is
        # listed here for convienience.
        # Example: holding register 60 becomes 30061.
        address_name = 30001 + starting_address
        self.puti(3, 'address',
            'Read starting at address 0x{:X} / {:d}'.format(starting_address,
                                                            address_name))

        self.puti(5, 'length', 'Read {:d} units of data'.format(self.half_word(4)))

        starting_address = self.half_word(6)
        self.puti(7, 'address',
            'Write starting at address 0x{:X} / {:d}'.format(starting_address,
                                                             address_name))

        quantity_of_outputs = self.half_word(8)
        self.puti(9, 'length',
                           'Write {} registers'.format(quantity_of_outputs))
        proper_bytecount = quantity_of_outputs * 2

        bytecount = self.data[10].data
        if bytecount == proper_bytecount:
            self.puti(10, 'length', 'Byte count: {}'.format(bytecount))
        else:
            self.puti(10, 'error',
                'Bad byte count, is {}, should be {}'.format(bytecount,
                                                             proper_bytecount))
        self.mimumum_length = bytecount + 13

        self.putl('data', 'Data, value 0x{:02X}', 10 + bytecount)

        self.check_crc(bytecount + 12)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'modbus'
    name = 'Modbus'
    longname = 'Modbus RTU over RS232/RS485'
    desc = 'Modbus RTU protocol for industrial applications.'
    license = 'gplv3+'
    inputs = ['uart']
    outputs = ['modbus']
    tags = ['Embedded/industrial']
    annotations = (
        ('sc-server-id', ''),
        ('sc-function', ''),
        ('sc-crc', ''),
        ('sc-address', ''),
        ('sc-data', ''),
        ('sc-length', ''),
        ('sc-error', ''),
        ('cs-server-id', ''),
        ('cs-function', ''),
        ('cs-crc', ''),
        ('cs-address', ''),
        ('cs-data', ''),
        ('cs-length', ''),
        ('cs-error', ''),
        ('error-indication', ''),
    )
    annotation_rows = (
        ('sc', 'Server->client', (0, 1, 2, 3, 4, 5, 6)),
        ('cs', 'Client->server', (7, 8, 9, 10, 11, 12, 13)),
        ('error-indicator', 'Errors in frame', (14,)),
    )
    options = (
        {'id': 'scchannel', 'desc': 'Server -> client channel',
            'default': rxtx_channels[0], 'values': rxtx_channels},
        {'id': 'cschannel', 'desc': 'Client -> server channel',
            'default': rxtx_channels[1], 'values': rxtx_channels},
        {'id': 'framegap', 'desc': 'Inter-frame bit gap', 'default': 28},
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.ADUSc = None # Start off with empty slave -> client ADU.
        self.ADUCs = None # Start off with empty client -> slave ADU.

        # The reason we have both (despite not supporting full duplex comms) is
        # because we want to be able to decode the message as both client ->
        # server and server -> client, and let the user see which of the two
        # the ADU was.

        self.bitlength = None # We will later test how long a bit is.

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def puta(self, start, end, ann_str, message):
        '''Put an annotation from start to end, with ann as a
        string. This means you don't have to know the ann's
        number to write annotations to it.'''
        ann = [s[0] for s in self.annotations].index(ann_str)
        self.put(start, end, self.out_ann, [ann, [message]])

    def decode_adu(self, ss, es, data, direction):
        '''Decode the next byte or bit (depending on type) in the ADU.
        ss: Start time of the data
        es: End time of the data
        data: Data as passed from the UART decoder
        direction: Is this data for the Cs (client -> server) or Sc (server ->
                   client) being decoded right now?'''
        ptype, rxtx, pdata = data

        # We don't have a nice way to get the baud rate from UART, so we have
        # to figure out how long a bit lasts. We do this by looking at the
        # length of (probably) the startbit.
        if self.bitlength is None:
            if ptype == 'STARTBIT' or ptype == 'STOPBIT':
                self.bitlength = es - ss
            else:
                # If we don't know the bitlength yet, we can't start decoding.
                return

        # Select the ADU, create the ADU if needed.
        # We set ADU.startNewFrame = True when we know the old one is over.
        if direction == 'Sc':
            if (self.ADUSc is None) or self.ADUSc.startNewFrame:
                self.ADUSc = Modbus_ADU_SC(self, ss, TX, 'sc-')
            ADU = self.ADUSc
        if direction == 'Cs':
            if self.ADUCs is None or self.ADUCs.startNewFrame:
                self.ADUCs = Modbus_ADU_CS(self, ss, TX, 'cs-')
            ADU = self.ADUCs

        # We need to determine if the last ADU is over.
        # According to the Modbus spec, there should be 3.5 characters worth of
        # space between each message. But if within a message there is a length
        # of more than 1.5 character, that's an error. For our purposes
        # somewhere between seems fine.
        # A character is 11 bits long, so (3.5 + 1.5)/2 * 11 ~= 28
        # TODO: Display error for too short or too long.
        if (ss - ADU.last_read) <= self.bitlength * self.options['framegap']:
            ADU.add_data(ss, es, data)
        else:
            # It's been too long since the last part of the ADU!
            # If there is any data in the ADU we need to show it to the user
            if len(ADU.data) > 0:
                # Extend errors for 3 bits after last byte, we can guarantee
                # space.
                ADU.close(ADU.data[-1].end + self.bitlength * 3)

            ADU.startNewFrame = True
            # Restart this function, it will make a new ADU for us.
            self.decode_adu(ss, es, data, direction)

    def decode(self, ss, es, data):
        ptype, rxtx, pdata = data

        # Ignore unknown/unsupported ptypes.
        if ptype not in ('STARTBIT', 'DATA', 'STOPBIT'):
            return

        # Decide what ADU(s) we need this packet to go to.
        # Note that it's possible to go to both ADUs.
        if rxtx_channels[rxtx] == self.options['scchannel']:
            self.decode_adu(ss, es, data, 'Sc')
        if rxtx_channels[rxtx] == self.options['cschannel']:
            self.decode_adu(ss, es, data, 'Cs')



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Fabian J. Stumpf <sigrok@fabianstumpf.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class Decoder(srd.Decoder):
    api_version = 3
    id = 'dmx512'
    name = 'DMX512'
    longname = 'Digital MultipleX 512'
    desc = 'Digital MultipleX 512 (DMX512) lighting protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Embedded/industrial', 'Lighting']
    channels = (
        {'id': 'dmx', 'name': 'DMX data', 'desc': 'Any DMX data line'},
    )
    annotations = (
        ('bit', 'Bit'),
        ('break', 'Break'),
        ('mab', 'Mark after break'),
        ('startbit', 'Start bit'),
        ('stopbits', 'Stop bit'),
        ('startcode', 'Start code'),
        ('channel', 'Channel'),
        ('interframe', 'Interframe'),
        ('interpacket', 'Interpacket'),
        ('data', 'Data'),
        ('error', 'Error'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0, 3, 4)),
        ('data', 'Data', (9,)),
        ('name', 'Logical', (1, 2, 5, 6, 7, 8)),
        ('errors', 'Errors', (10,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.sample_usec = None
        self.run_start = -1
        self.run_bit = 0
        self.state = 'FIND BREAK'

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
            self.sample_usec = 1 / value * 1000000
            self.skip_per_bit = int(4 / self.sample_usec)

    def putr(self, data):
        self.put(self.run_start, self.samplenum, self.out_ann, data)

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        while True:
            # Seek for an interval with no state change with a length between
            # 88 and 1000000 us (BREAK).
            if self.state == 'FIND BREAK':
                (dmx,) = self.wait({0: 'h' if self.run_bit == 0 else 'l'})
                runlen = (self.samplenum - self.run_start) * self.sample_usec
                if runlen > 88 and runlen < 1000000:
                    self.putr([1, ['Break']])
                    self.bit_break = self.run_bit
                    self.state = 'MARK MAB'
                    self.channel = 0
                elif runlen >= 1000000:
                    # Error condition.
                    self.putr([10, ['Invalid break length']])
                self.run_bit = dmx
                self.run_start = self.samplenum
            # Directly following the BREAK is the MARK AFTER BREAK.
            elif self.state == 'MARK MAB':
                (dmx,) = self.wait({0: 'h' if self.run_bit == 0 else 'l'})
                self.putr([2, ['MAB']])
                self.state = 'READ BYTE'
                self.channel = 0
                self.bit = 0
                self.aggreg = dmx
                self.run_start = self.samplenum
            # Mark and read a single transmitted byte
            # (start bit, 8 data bits, 2 stop bits).
            elif self.state == 'READ BYTE':
                (dmx,) = self.wait()
                self.next_sample = self.run_start + (self.bit + 1) * self.skip_per_bit
                self.aggreg += dmx
                if self.samplenum != self.next_sample:
                    continue
                bit_value = 0 if round(self.aggreg/self.skip_per_bit) == self.bit_break else 1

                if self.bit == 0:
                    self.byte = 0
                    self.putr([3, ['Start bit']])
                    if bit_value != 0:
                        # (Possibly) invalid start bit, mark but don't fail.
                        self.put(self.samplenum, self.samplenum,
                                 self.out_ann, [10, ['Invalid start bit']])
                elif self.bit >= 9:
                    self.put(self.samplenum - self.skip_per_bit,
                        self.samplenum, self.out_ann, [4, ['Stop bit']])
                    if bit_value != 1:
                        # Invalid stop bit, mark.
                        self.put(self.samplenum, self.samplenum,
                            self.out_ann, [10, ['Invalid stop bit']])
                        if self.bit == 10:
                            # On invalid 2nd stop bit, search for new break.
                            self.run_bit = dmx
                            self.state = 'FIND BREAK'
                else:
                    # Label and process one bit.
                    self.put(self.samplenum - self.skip_per_bit,
                        self.samplenum, self.out_ann, [0, [str(bit_value)]])
                    self.byte |= bit_value << (self.bit - 1)

                # Label a complete byte.
                if self.bit == 10:
                    if self.channel == 0:
                        d = [5, ['Start code']]
                    else:
                        d = [6, ['Channel ' + str(self.channel)]]
                    self.put(self.run_start, self.next_sample, self.out_ann, d)
                    self.put(self.run_start + self.skip_per_bit,
                        self.next_sample - 2 * self.skip_per_bit,
                        self.out_ann, [9, [str(self.byte) + ' / ' + \
                        str(hex(self.byte))]])
                    # Continue by scanning the IFT.
                    self.channel += 1
                    self.run_start = self.samplenum
                    self.run_bit = dmx
                    self.state = 'MARK IFT'

                self.aggreg = dmx
                self.bit += 1
            # Mark the INTERFRAME-TIME between bytes / INTERPACKET-TIME between packets.
            elif self.state == 'MARK IFT':
                (dmx,) = self.wait({0: 'h' if self.run_bit == 0 else 'l'})
                if self.channel > 512:
                    self.putr([8, ['Interpacket']])
                    self.state = 'FIND BREAK'
                    self.run_bit = dmx
                    self.run_start = self.samplenum
                else:
                    self.putr([7, ['Interframe']])
                    self.state = 'READ BYTE'
                    self.bit = 0
                    self.run_start = self.samplenum



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Shirow Miura <shirowmiura@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

symbol_map = {
    0b0000: '0',
    0b1000: '1',
    0b0100: '2',
    0b1100: '3',
    0b0010: '4',
    0b1010: '5',
    0b0110: '6',
    0b1110: '7',
    0b0001: '8',
    0b1001: '9',
    0b0101: 'A',
    0b1101: 'C',
    0b0011: 'F',
    0b1011: 'H',
    0b0111: 'P',
    0b1111: 'U',
}

START, STOP, CLOCK, DATA = range(4)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'signature'
    name = 'Signature'
    longname = 'Signature analysis'
    desc = 'Annotate signature of logic patterns.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Debug/trace', 'Util', 'Encoding']
    channels = (
        {'id': 'start', 'name': 'START', 'desc': 'START channel'},
        {'id': 'stop', 'name': 'STOP', 'desc': 'STOP channel'},
        {'id': 'clk', 'name': 'CLOCK', 'desc': 'CLOCK channel'},
        {'id': 'data', 'name': 'DATA', 'desc': 'DATA channel'},
    )
    options = (
        {'id': 'start_edge', 'desc': 'START edge polarity',
            'default': 'rising', 'values': ('rising', 'falling')},
        {'id': 'stop_edge', 'desc': 'STOP edge polarity',
            'default': 'rising', 'values': ('rising', 'falling')},
        {'id': 'clk_edge', 'desc': 'CLOCK edge polarity',
            'default': 'falling', 'values': ('rising', 'falling')},
        {'id': 'annbits', 'desc': 'Enable bit level annotations',
            'default': 'no', 'values': ('yes', 'no')},
    )
    annotations = (
        ('bit0', 'Bit0'),
        ('bit1', 'Bit1'),
        ('start', 'START'),
        ('stop', 'STOP'),
        ('sig', 'Sig')
    )
    annotation_rows = (
        ('bits', 'Bits', (0, 1, 2, 3)),
        ('sig', 'Sig', (4,))
    )

    def __init__(self):
        self.reset()

    def reset(self):
        pass

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putsig(self, ss, es, signature):
        s = ''.join([symbol_map[(signature >>  0) & 0x0f],
                     symbol_map[(signature >>  4) & 0x0f],
                     symbol_map[(signature >>  8) & 0x0f],
                     symbol_map[(signature >> 12) & 0x0f]])
        self.put(ss, es, self.out_ann, [4, [s]])

    def putb(self, ss, ann):
        self.put(ss, self.samplenum, self.out_ann, ann)

    def decode(self):
        opt = self.options
        start_edge_mode_rising = opt['start_edge'] == 'rising'
        stop_edge_mode_rising = opt['stop_edge'] == 'rising'
        annbits = opt['annbits'] == 'yes'
        gate_is_open = False
        sample_start = None
        started = False
        last_samplenum = 0
        prev_start = 0 if start_edge_mode_rising else 1
        prev_stop = 0 if stop_edge_mode_rising else 1
        shiftreg = 0

        while True:
            start, stop, _, data = self.wait({CLOCK: opt['clk_edge']})
            if start != prev_start and not gate_is_open:
                gate_is_open = (start == 1) if start_edge_mode_rising else (start == 0)
                if gate_is_open:
                    # Start sampling.
                    sample_start = self.samplenum
                    started = True
            elif stop != prev_stop and gate_is_open:
                gate_is_open = not ((stop == 1) if stop_edge_mode_rising else (stop == 0))
                if not gate_is_open:
                    # Stop sampling.
                    if annbits:
                        self.putb(last_samplenum, [3, ['STOP', 'STP', 'P']])
                    self.putsig(sample_start, self.samplenum, shiftreg)
                    shiftreg = 0
                    sample_start = None
            if gate_is_open:
                if annbits:
                    if started:
                        s = '<{}>'.format(data)
                        self.putb(last_samplenum, [2, ['START' + s, 'STR' + s, 'S' + s]])
                        started = False
                    else:
                        self.putb(last_samplenum, [data, [str(data)]])
                incoming = (bin(shiftreg & 0b0000_0010_1001_0001).count('1') + data) & 1
                shiftreg = (incoming << 15) | (shiftreg >> 1)
            prev_start = start
            prev_stop = stop
            last_samplenum = self.samplenum



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Vladimir Ermakov <vooon341@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from functools import reduce

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'rgb_led_ws281x'
    name = 'RGB LED (WS281x)'
    longname = 'RGB LED string decoder (WS281x)'
    desc = 'RGB LED string protocol (WS281x).'
    license = 'gplv3+'
    inputs = ['logic']
    outputs = []
    tags = ['Display', 'IC']
    channels = (
        {'id': 'din', 'name': 'DIN', 'desc': 'DIN data line'},
    )
    annotations = (
        ('bit', 'Bit'),
        ('reset', 'RESET'),
        ('rgb', 'RGB'),
    )
    annotation_rows = (
        ('bit', 'Bits', (0, 1)),
        ('rgb', 'RGB', (2,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.oldpin = None
        self.ss_packet = None
        self.ss = None
        self.es = None
        self.bits = []
        self.inreset = False

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def handle_bits(self, samplenum):
        if len(self.bits) == 24:
            grb = reduce(lambda a, b: (a << 1) | b, self.bits)
            rgb = (grb & 0xff0000) >> 8 | (grb & 0x00ff00) << 8 | (grb & 0x0000ff)
            self.put(self.ss_packet, samplenum, self.out_ann,
                     [2, ['#%06x' % rgb]])
            self.bits = []
            self.ss_packet = None

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        while True:
            # TODO: Come up with more appropriate self.wait() conditions.
            (pin,) = self.wait()

            if self.oldpin is None:
                self.oldpin = pin
                continue

            # Check RESET condition (manufacturer recommends 50 usec minimal,
            # but real minimum is ~10 usec).
            if not self.inreset and not pin and self.es is not None and \
                    self.ss is not None and \
                    (self.samplenum - self.es) / self.samplerate > 50e-6:

                # Decode last bit value.
                tH = (self.es - self.ss) / self.samplerate
                bit_ = True if tH >= 625e-9 else False

                self.bits.append(bit_)
                self.handle_bits(self.es)

                self.put(self.ss, self.es, self.out_ann, [0, ['%d' % bit_]])
                self.put(self.es, self.samplenum, self.out_ann,
                         [1, ['RESET', 'RST', 'R']])

                self.inreset = True
                self.bits = []
                self.ss_packet = None
                self.ss = None

            if not self.oldpin and pin:
                # Rising edge.
                if self.ss and self.es:
                    period = self.samplenum - self.ss
                    duty = self.es - self.ss
                    # Ideal duty for T0H: 33%, T1H: 66%.
                    bit_ = (duty / period) > 0.5

                    self.put(self.ss, self.samplenum, self.out_ann,
                             [0, ['%d' % bit_]])

                    self.bits.append(bit_)
                    self.handle_bits(self.samplenum)

                if self.ss_packet is None:
                    self.ss_packet = self.samplenum

                self.ss = self.samplenum

            elif self.oldpin and not pin:
                # Falling edge.
                self.inreset = False
                self.es = self.samplenum

            self.oldpin = pin



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Gerhard Sittig <gerhard.sittig@gmx.net>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
PDI (Program and Debug Interface) is an Atmel proprietary interface for
external programming and on-chip debugging of the device.

See the Atmel Application Note AVR1612 "PDI programming driver" and the
"Program and Debug Interface" section in the Xmega A manual for details.

The protocol uses two pins: the RESET pin and one dedicated DATA pin.
The RESET pin provides a clock, the DATA pin communicates serial frames
with a start bit, eight data bits, an even parity bit, and two stop bits.
Data communication is bidirectional and half duplex, the device will
provide response data after reception of a respective request.

Protocol frames communicate opcodes and their arguments, which provides
random and sequential access to the device's address space. By accessing
the registers of internal peripherals, especially the NVM controller,
it's possible to identify the device, read from and write to several
kinds of memory (signature rows, fuses and lock bits, internal flash and
EEPROM, memory mapped peripherals), and to control execution of software
on the device.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# TODO: Better support for various LM75 compatible devices.

import sigrokdecode as srd

# LM75 only supports 9 bit resolution, compatible devices usually 9-12 bits.
resolution = {
    # CONFIG[6:5]: <resolution>
    0x00: 9,
    0x01: 10,
    0x02: 11,
    0x03: 12,
}

ft = {
    # CONFIG[4:3]: <fault tolerance setting>
    0x00: 1,
    0x01: 2,
    0x02: 4,
    0x03: 6,
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'lm75'
    name = 'LM75'
    longname = 'National LM75'
    desc = 'National LM75 (and compatibles) temperature sensor.'
    license = 'gplv2+'
    inputs = ['i2c']
    outputs = []
    tags = ['Sensor']
    options = (
        {'id': 'sensor', 'desc': 'Sensor type', 'default': 'lm75',
            'values': ('lm75',)},
        {'id': 'resolution', 'desc': 'Resolution (bits)', 'default': 9,
            'values': (9, 10, 11, 12)},
    )
    annotations = (
        ('celsius', 'Temperature in degrees Celsius'),
        ('kelvin', 'Temperature in Kelvin'),
        ('text-verbose', 'Human-readable text (verbose)'),
        ('text', 'Human-readable text'),
        ('warnings', 'Human-readable warnings'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.reg = 0x00 # Currently selected register
        self.databytes = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        # Helper for annotations which span exactly one I²C packet.
        self.put(self.ss, self.es, self.out_ann, data)

    def putb(self, data):
        # Helper for annotations which span a block of I²C packets.
        self.put(self.ss_block, self.es_block, self.out_ann, data)

    def warn_upon_invalid_slave(self, addr):
        # LM75 and compatible devices have a 7-bit I²C slave address where
        # the 4 MSBs are fixed to 1001, and the 3 LSBs are configurable.
        # Thus, valid slave addresses (1001xxx) range from 0x48 to 0x4f.
        if addr not in range(0x48, 0x4f + 1):
            s = 'Warning: I²C slave 0x%02x not an LM75 compatible sensor.'
            self.putx([4, [s % addr]])

    def output_temperature(self, s, rw):
        # TODO: Support for resolutions other than 9 bit.
        before, after = self.databytes[0], (self.databytes[1] >> 7) * 5
        celsius = float('%d.%d' % (before, after))
        kelvin = celsius + 273.15
        self.putb([0, ['%s: %.1f °C' % (s, celsius)]])
        self.putb([1, ['%s: %.1f K' % (s, kelvin)]])

        # Warn about the temperature register (0x00) being read-only.
        if s == 'Temperature' and rw == 'WRITE':
            s = 'Warning: The temperature register is read-only!'
            self.putb([4, [s]])

    def handle_temperature_reg(self, b, s, rw):
        # Common helper for the temperature/T_HYST/T_OS registers.
        if len(self.databytes) == 0:
            self.ss_block = self.ss
            self.databytes.append(b)
            return
        self.databytes.append(b)
        self.es_block = self.es
        self.output_temperature(s, rw)
        self.databytes = []

    def handle_reg_0x00(self, b, rw):
        # Temperature register (16bits, read-only).
        self.handle_temperature_reg(b, 'Temperature', rw)

    def handle_reg_0x01(self, b, rw):
        # Configuration register (8 bits, read/write).
        # TODO: Bit-exact annotation ranges.

        sd = b & (1 << 0)
        tmp = 'normal operation' if (sd == 0) else 'shutdown mode'
        s = 'SD = %d: %s\n' % (sd, tmp)
        s2 = 'SD = %s, ' % tmp

        cmp_int = b & (1 << 1)
        tmp = 'comparator' if (cmp_int == 0) else 'interrupt'
        s += 'CMP/INT = %d: %s mode\n' % (cmp_int, tmp)
        s2 += 'CMP/INT = %s, ' % tmp

        pol = b & (1 << 2)
        tmp = 'low' if (pol == 0) else 'high'
        s += 'POL = %d: OS polarity is active-%s\n' % (pol, tmp)
        s2 += 'POL = active-%s, ' % tmp

        bits = (b & ((1 << 4) | (1 << 3))) >> 3
        s += 'Fault tolerance setting: %d bit(s)\n' % ft[bits]
        s2 += 'FT = %d' % ft[bits]

        # Not supported by LM75, but by various compatible devices.
        if self.options['sensor'] != 'lm75': # TODO
            bits = (b & ((1 << 6) | (1 << 5))) >> 5
            s += 'Resolution: %d bits\n' % resolution[bits]
            s2 += ', resolution = %d' % resolution[bits]

        self.putx([2, [s]])
        self.putx([3, [s2]])

    def handle_reg_0x02(self, b, rw):
        # T_HYST register (16 bits, read/write).
        self.handle_temperature_reg(b, 'T_HYST trip temperature', rw)

    def handle_reg_0x03(self, b, rw):
        # T_OS register (16 bits, read/write).
        self.handle_temperature_reg(b, 'T_OS trip temperature', rw)

    def decode(self, ss, es, data):
        cmd, databyte = data

        # Store the start/end samples of this I²C packet.
        self.ss, self.es = ss, es

        # State machine.
        if self.state == 'IDLE':
            # Wait for an I²C START condition.
            if cmd != 'START':
                return
            self.state = 'GET SLAVE ADDR'
        elif self.state == 'GET SLAVE ADDR':
            # Wait for an address read/write operation.
            if cmd in ('ADDRESS READ', 'ADDRESS WRITE'):
                self.warn_upon_invalid_slave(databyte)
                self.state = cmd[8:] + ' REGS' # READ REGS / WRITE REGS
        elif self.state in ('READ REGS', 'WRITE REGS'):
            if cmd in ('DATA READ', 'DATA WRITE'):
                handle_reg = getattr(self, 'handle_reg_0x%02x' % self.reg)
                handle_reg(databyte, cmd[5:]) # READ / WRITE
            elif cmd == 'STOP':
                # TODO: Any output?
                self.state = 'IDLE'
            else:
                # self.putx([0, ['Ignoring: %s (data=%s)' % (cmd, databyte)]])
                pass



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Matt Ranostay <mranostay@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class Decoder(srd.Decoder):
    api_version = 3
    id = 'rgb_led_spi'
    name = 'RGB LED (SPI)'
    longname = 'RGB LED string decoder (SPI)'
    desc = 'RGB LED string protocol (RGB values clocked over SPI).'
    license = 'gplv2+'
    inputs = ['spi']
    outputs = []
    tags = ['Display']
    annotations = (
        ('rgb', 'RGB values'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.ss_cmd, self.es_cmd = 0, 0
        self.mosi_bytes = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss_cmd, self.es_cmd, self.out_ann, data)

    def decode(self, ss, es, data):
        ptype, mosi, miso = data

        # Only care about data packets.
        if ptype != 'DATA':
            return
        self.ss, self.es = ss, es

        if len(self.mosi_bytes) == 0:
            self.ss_cmd = ss
        self.mosi_bytes.append(mosi)

        # RGB value == 3 bytes
        if len(self.mosi_bytes) != 3:
            return

        red, green, blue = self.mosi_bytes
        rgb_value = int(red) << 16 | int(green) << 8 | int(blue)

        self.es_cmd = es
        self.putx([0, ['#%.6x' % rgb_value]])
        self.mosi_bytes = []



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Johannes Roemer <jroemer@physik.uni-wuerzburg.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
This decoder handles the proprietary single wire communication protocol used
by the Aosong AM230x/DHTxx/RHTxx series of digital humidity and temperature
sensors.

Sample rate:
A sample rate of at least 200kHz is recommended to properly detect all the
elements of the protocol.

Options:
The AM230x and DHTxx/RHTxx digital humidity and temperature sensors use the
same single-wire protocol with different encoding of the measured values.
Therefore the option 'device' must be used to properly decode the
communication of the respective sensor.
'''

from .pd import Decoder
